/***************************************************************************
 *   Copyright (C) 2005 by s4kk3   										   *
 *   s4kk33@gmail.com   												   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "fileinfo.h"

using namespace std;

FileInfo	*FileInfo::info	=	NULL;
FileInfo	*FileInfo::getInstance( FileView *fv )
{ 
	if( !info )
		info	=	new FileInfo( fv );
	
	return info; 
}

//-------------------------------------------------------------------------
// constructor
FileInfo::FileInfo( FileView *f )
{
	info	=	this;
	fv	=	f;
	mw	=	MainWindow::getInstance();
		
	window		=	gtk_window_new( GTK_WINDOW_TOPLEVEL );
	fileBook	=	GTK_NOTEBOOK( gtk_notebook_new() );

	GtkVBox *box	=	GTK_VBOX( gtk_vbox_new( false,	0 ) );

	gtk_notebook_set_tab_pos( fileBook,	GTK_POS_LEFT );
	gtk_notebook_set_show_border( fileBook,	false );
	gtk_notebook_set_tab_vborder( fileBook, 0 );
	gtk_notebook_set_tab_hborder( fileBook, 0 );

	cancelButton	=	GTK_BUTTON( gtk_button_new_from_stock( GTK_STOCK_CANCEL ) );
	okButton	=	GTK_BUTTON( gtk_button_new_from_stock( GTK_STOCK_OK ) );

	GtkWidget *buttonBox	=	gtk_hbutton_box_new();

	gtk_box_pack_end( GTK_BOX( buttonBox ),	GTK_WIDGET( okButton ),		false,	false,	5 );
	gtk_box_pack_end( GTK_BOX( buttonBox ),	GTK_WIDGET( cancelButton ),	false,	false,	5 );
	gtk_button_box_set_layout( GTK_BUTTON_BOX( buttonBox ), GTK_BUTTONBOX_END );
	gtk_box_set_spacing( GTK_BOX( buttonBox ), 5);

	gtk_box_pack_start( GTK_BOX( box ),	GTK_WIDGET( fileBook ),	true,	true,	0 );
	gtk_box_pack_start( GTK_BOX( box ),	buttonBox,		true,	true,	0 );
	gtk_container_add( GTK_CONTAINER( window ), GTK_WIDGET( box ) );

	gtk_widget_show_all( window );

	cancelCallback.connect( G_OBJECT( cancelButton ),	"clicked",	this,	
				&FileInfo::cancelClicked,		NULL );
	okCallback.connect( G_OBJECT( okButton ),			"clicked",	this,
				&FileInfo::okClicked,			NULL );
	destroyCallback.connect( G_OBJECT( window ),		"destroy",	this,
				 &FileInfo::onDestroyEvent,	NULL );
}

//-------------------------------------------------------------------------
// destructor
FileInfo::~FileInfo()
{

}

//-------------------------------------------------------------------------
// setText method
void FileInfo::setEntry( string path )
{
	string	dataPath	=	mw->getGladeFile( "fileinfo.glade" );
	
	GladeXML	*xml	=	glade_xml_new( dataPath.c_str(),	NULL,	NULL );

	GtkWidget *win	=	glade_xml_get_widget( xml,	"window1" );

	infoBook	=	GTK_NOTEBOOK(	glade_xml_get_widget( xml,	"infoBook" ) );
	fileIcon	=	GTK_IMAGE(	glade_xml_get_widget( xml,	"image1" ) );
	nameEntry	=	GTK_ENTRY(	glade_xml_get_widget( xml,	"nameEntry" ) );
	actionButton	=	GTK_BUTTON(	glade_xml_get_widget( xml,	"button1" ) );

	locationLabel	=	GTK_LABEL( glade_xml_get_widget( xml,	"location" ) );
	mimeLabel	=	GTK_LABEL( glade_xml_get_widget( xml,	"mime" ) );
	sizeLabel	=	GTK_LABEL( glade_xml_get_widget( xml,	"size" ) );
	exactSizeLabel	=	GTK_LABEL( glade_xml_get_widget( xml,	"exact_size" ) );
	runActionLabel	=	GTK_LABEL( glade_xml_get_widget( xml,	"run_action" ) );

        gtk_widget_ref( GTK_WIDGET( infoBook ) );
        gtk_container_remove(GTK_CONTAINER( win ), GTK_WIDGET( infoBook ) );
        gtk_widget_destroy( win );

	string	name	=	g_path_get_basename( path.c_str() );
	string folder	=	g_path_get_dirname( path.c_str() );

	string lab	=	name;
	if(name.size()	>	7)
                lab = name.substr( 0,	7 )	+	"...";

	GtkWidget *label	=	gtk_label_new( lab.c_str() );
	gtk_notebook_append_page( fileBook,	GTK_WIDGET( infoBook ),	label );
	gtk_widget_show_all( GTK_WIDGET( fileBook ) );

	if( !Util::checkWritePermissions( folder ) )
		gtk_widget_set_sensitive( GTK_WIDGET( nameEntry ),	false );
	//maybe we need to check this one too, trivial. have to check later
	if( !Util::checkWritePermissions( folder ) )
		gtk_widget_set_sensitive( GTK_WIDGET( nameEntry ),	false );

	const char	*mime	=	xdg_mime_get_mime_type_for_file( path.c_str() );
	
	double	exactSize;
	
	if( g_file_test( path.c_str(),	G_FILE_TEST_IS_DIR ) )
		exactSize	=	Get::getFolderSize( path );
	
	else 
		exactSize	=	Get::getFileSize( path );
		
	string	size	=	Get::getSize( exactSize );

	if( mime	==	NULL	&&	g_str_has_suffix( path.c_str(),	".avi")) 
		mime	=	"video/x-msvideo";
	
	char	exact[1024];
	
	sprintf( exact,	"%.0f",	exactSize );
	
	string	action	=	fv->getDefaultRunAction( mime );

	if( !g_file_test( path.c_str(),	G_FILE_TEST_IS_DIR)	&&	action.size()	==	0 )
		action	=	"None specified yet";
	
	GdkPixbuf	*buf	=	mw->getIcon( path,	24,	false );

	gtk_image_set_from_pixbuf( fileIcon,	buf );
	gtk_entry_set_text( nameEntry,		name.c_str() );
	gtk_label_set_text( locationLabel,	g_path_get_dirname( path.c_str() ) );
	gtk_label_set_text( mimeLabel,		mime );
	gtk_label_set_text( sizeLabel,		size.c_str() );
	gtk_label_set_text( exactSizeLabel,	exact );
	gtk_label_set_text( runActionLabel,	action.c_str() );

	entryList[nameEntry] = path;

#if ENABLE_EXIF
	if( g_strrstr( mime,	"image" ) )
		this->setExifData( path );
#endif

	actionCallback.connect( G_OBJECT( actionButton ),	"clicked",	this,
				&FileInfo::setActionClicked,	NULL );
}

//-------------------------------------------------------------------------
// cancelClicked method
void FileInfo::cancelClicked( GtkWidget *widget,	gpointer data )
{
	gtk_widget_destroy( window );
}

//-------------------------------------------------------------------------
// okClicked method
void FileInfo::okClicked( GtkWidget *widget,	gpointer data )
{
	for( map<GtkEntry*, string>::iterator iter	=	entryList.begin(); 
					 iter	!=	entryList.end();
					 iter++ )
	{
		string oldPath	=	iter->second;
		string newPath	=	g_path_get_dirname( oldPath.c_str() )	+	(string)"/"	+
					(string)gtk_entry_get_text( iter->first );

		if( !Util::compare( oldPath,	newPath ) )
			rename( oldPath.c_str(),	newPath.c_str() );
	}
	gtk_widget_destroy( window );
}

//-------------------------------------------------------------------------
// setActionClicked method
void FileInfo::setActionClicked( GtkWidget *widget,	gpointer data )
{
	string mime	=	gtk_label_get_label( mimeLabel );
	string action	=	Dialog::setDefaultRunAction( mime,	MainWindow::getInstance()->getHomeDir() );
	gtk_label_set_text( runActionLabel,	action.c_str() );
}

//-------------------------------------------------------------------------
// setExifData method
#if ENABLE_EXIF
void FileInfo::setExifData( const std::string& file )
{
	GtkWidget *win		=	gtk_scrolled_window_new( false,	false );
	GtkTextTagTable *table	=	gtk_text_tag_table_new();
	GtkTextBuffer *buf	=	gtk_text_buffer_new( table );
	GtkWidget *view		=	gtk_text_view_new_with_buffer( buf );

	GtkTextIter iter;

	gtk_container_add( GTK_CONTAINER( win ),	view );

	list<string> data	=	Get::getExifData( file );
	
	list<string>::iterator it;

	gtk_text_buffer_get_iter_at_offset( buf,	&iter,	0 );

	 gtk_text_buffer_create_tag( buf,	"bold",	"weight",	PANGO_WEIGHT_BOLD,	NULL );  

	for( it	=	data.begin();
	     it	!=	data.end();
	     it++ )
	{
		string tmp	=	(*it);

		int pos		=	tmp.find( ":",	0 );
		string markup	=	tmp.substr( 0,	pos );
		tmp.erase( 0,	pos );

		tmp += "\n";

		gtk_text_buffer_insert_with_tags_by_name( buf,	&iter,	markup.c_str(),	markup.size(),	"bold",	NULL );
		gtk_text_buffer_insert( buf,	&iter,	tmp.c_str(), tmp.size() );
	}

	gtk_widget_show_all( win );

	gtk_notebook_append_page( infoBook,	win,	gtk_label_new( "EXIF data" ) );
}
#endif

void FileInfo::onDestroyEvent( GtkWidget *widget,	gpointer data )
{
	delete info;
	info	=	NULL;
}
