/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#if ENABLE_ICONVIEW
	#include "gtkiconview.h"
#else
	#include <gtk/gtkiconview.h>
#endif

#include <gdk/gdkkeysyms.h>
#include <ctime>

#include "mainwindow.h"

using namespace std;

MainWindow *MainWindow::mainWin	=	NULL;
MainWindow *MainWindow::getInstance(const char *path, const string& exe)
{ 
	if( !mainWin )
		mainWin	=	new	MainWindow( path,	exe ); 
	
	return mainWin; 
}

//-------------------------------------------------------------------------
// constructor
MainWindow::MainWindow( const char *path,	const string& executable )
{		
	mainWin		=	this;
	thumb		=	Thumbnail::getInstance();
	exec			=	executable;
	
	string dataPath	=	this->getGladeFile( "mainwindow.glade" );
	
	homeDir		=	g_get_home_dir();
	xml		=	glade_xml_new( dataPath.c_str(), 0, 0 );

	//load widgets
	quitItem	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"quit1" ) );
	showHiddenItem	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"show_hidden_files" ) );
	openTab		=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"open_new_tab1" ) );
	open_Tab	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"open_new_tab2" ) );
	renameItem	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"rename_tab" ) );
	closeTabItem	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"close_tab" ) );
	newTerminalItem	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"open_new_terminal" ) );
	openSettings	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"settings1" ) );
	changelogItem	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"changelog" ) );
	showBottomBar	=	GTK_MENU_ITEM( glade_xml_get_widget( xml,	"show_bottom_toolbar1" ) );
	
	iconView	=	GTK_RADIO_MENU_ITEM( glade_xml_get_widget( xml,	"icon_view" ) );
	listView	=	GTK_RADIO_MENU_ITEM( glade_xml_get_widget( xml,	"list_view" ) );
	imageView	=	GTK_RADIO_MENU_ITEM( glade_xml_get_widget( xml,	"image_view" ) );

	parentButton	=	GTK_TOOL_BUTTON( glade_xml_get_widget( xml,	"parentButton" ) );
	nextButton	=	GTK_TOOL_BUTTON( glade_xml_get_widget( xml,	"nextButton" ) );
	goButton	=	GTK_TOOL_BUTTON( glade_xml_get_widget( xml,	"toolbutton3" ) );
	prevButton	=	GTK_TOOL_BUTTON( glade_xml_get_widget( xml,	"previousButton" ) );

	pathEntry	=	GTK_ENTRY( glade_xml_get_widget( xml,	"pathEntry" ) );

	dboardWindow	=	GTK_SCROLLED_WINDOW( glade_xml_get_widget( xml,	"expanderwindow" ) );

	book		=	GTK_NOTEBOOK( glade_xml_get_widget( xml,	"book" ) );

	mainPaned	=	GTK_PANED( glade_xml_get_widget( xml,	"hpaned1" ) );

	mainBox		=	glade_xml_get_widget( xml,	"vbox1" );
	tabMenu		=	glade_xml_get_widget( xml,	"menu6" );
	bottomBar	=	glade_xml_get_widget( xml,	"bottomBar" );
	sidebarBook	=	glade_xml_get_widget( xml,	"notebook1" );

	mainWindow	=	GTK_WINDOW( glade_xml_get_widget( xml,	"mainWindow" ) );

	mainStatus		=	GTK_STATUSBAR( glade_xml_get_widget( xml,	"statusbar1" ) );
	itemCountStatus		=	GTK_STATUSBAR( glade_xml_get_widget( xml,	"statusbar2" ) );
	hiddenCountStatus	=	GTK_STATUSBAR( glade_xml_get_widget( xml,	"statusbar3" ) );

	mainToolBar	=	GTK_TOOLBAR( glade_xml_get_widget( xml, "toolbar1" ) );

	//keybindings
	GtkMenu	*menu	=	GTK_MENU( glade_xml_get_widget( xml,	"menu7" ) );
	group	=	gtk_accel_group_new();
	gtk_window_add_accel_group( GTK_WINDOW( mainWindow ),	group );
	gtk_menu_set_accel_group( menu,	group );
	
	GtkWidget	*parentItem	=	glade_xml_get_widget( xml,	"parent1" );
	gtk_widget_add_accelerator( parentItem,	"activate",			group,	
								GDK_s,		GDK_CONTROL_MASK,	GTK_ACCEL_MASK );

	GtkWidget	*prevItem	=	glade_xml_get_widget( xml,	"prev1" );
	gtk_widget_add_accelerator( prevItem,	"activate",			group,
								GDK_a,		GDK_CONTROL_MASK,	GTK_ACCEL_MASK );

	GtkWidget	*nextItem	=	glade_xml_get_widget( xml,	"next1" );
	gtk_widget_add_accelerator( nextItem,	"activate",			group,
								GDK_d,		GDK_CONTROL_MASK,	GTK_ACCEL_MASK );

	//connect keybindings
	parentCallbackAcc.connect	( G_OBJECT( parentItem ),	"activate", this, &MainWindow::parentClicked,	0 );
	prevCallbackAcc.connect		( G_OBJECT( prevItem ),		"activate",	this, &MainWindow::prevClicked,		0 );
	nextCallbackAcc.connect		( G_OBJECT( nextItem ),		"activate", this, &MainWindow::nextClicked,		0 );	

	id	=	0;

	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( iconView ),	false );
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( listView ),	false );
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( imageView ),	false );

	//config checks
	string	configDir	=	(string)homeDir	+	"/.gbrowser";
	if( !g_file_test( configDir.c_str(),	G_FILE_TEST_EXISTS ) )
	{
		string	cmd	=	"mkdir "	+	configDir;
		system( cmd.c_str() );
	}

	valuesFile	=	configDir	+	"/valuesFile";

	if( !g_file_test( valuesFile.c_str(),	G_FILE_TEST_EXISTS ) )
	{
		string	cmd	=	"touch "	+	valuesFile;
		system( cmd.c_str() );

		showHidden	=	false;
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( showBottomBar ),	true );
		
		this->setSettings( true );
	}

	string	actionsFile	=	configDir	+	"/actions";
	if( !g_file_test( actionsFile.c_str(),	G_FILE_TEST_EXISTS ) )
	{
		string	cmd	=	"touch "	+	actionsFile;
		system( cmd.c_str() );
		
		GKeyFile*	values	=	g_key_file_new();

		g_key_file_set_string( values,	"application/x-executable",	"Default",		"bash -c" );
		g_key_file_set_boolean( values,	"application/x-executable",	"bash -c",		FALSE );
	
		ofstream	val( actionsFile.c_str() );
		char	*v	=	g_key_file_to_data( values, 0,	0 );
		val	<<	v;
		val.close();
		g_free( v );
		g_key_file_free( values );
	}

	string bookmarkFile	=	configDir	+	"/bookmarks";
	if( !g_file_test( bookmarkFile.c_str(),	G_FILE_TEST_EXISTS ) )
	{
		string	cmd	=	"touch "	+	bookmarkFile;
		system( cmd.c_str() );
		
		GKeyFile*	values	=	g_key_file_new();

		g_key_file_set_string( values,	"Home",	"Path",	homeDir );
	
		ofstream	val( bookmarkFile.c_str() );
		char	*v	=	g_key_file_to_data( values, 0,	0 );
		val	<<	v;
		val.close();

		g_free( v );
		g_key_file_free( values );
	}
	
	allowNext	=	false;
	allowPrev	=	false;

	this->getSettings();

	gtk_widget_set_sensitive( GTK_WIDGET( nextButton ),	false );
	gtk_widget_set_sensitive( GTK_WIDGET( prevButton ),	false );
	
	if( !g_file_test( path, G_FILE_TEST_EXISTS ) ) 
		path	=	this->getStringValue( "Values",	"defaultPath" );

	gtk_entry_set_text( pathEntry,	path );
	gtk_notebook_remove_page( book,	0 );

	if( Util::compare( themeName,	"System Default" ) )
		theme	=	gtk_icon_theme_get_default();
	else
	{
		theme	=	gtk_icon_theme_new();
		gtk_icon_theme_set_custom_theme( theme,	themeName.c_str() );
	}
	
	Sidebar::getInstance();
	Dispatch::getInstance();
	
	for( int i = 0; i != this->getIntValue( "Values",	"tabsOpened" ); i++ ) 
		this->addNewTab( path );
	
	quitCallback.connect( G_OBJECT( quitItem ),			"activate",		this,	
					&MainWindow::quitClicked,		0 );
	quitCallback.connect( G_OBJECT( mainWindow ),			"destroy",		this,	
					&MainWindow::quitClicked,		0 );
	parentCallback.connect( G_OBJECT( parentButton ),		"clicked",		this,	
					&MainWindow::parentClicked,		0 );
	goCallback.connect( G_OBJECT( goButton ),			"clicked",		this,	
					&MainWindow::goClicked,			0 );
	goCallback.connect( G_OBJECT( pathEntry ),			"activate",		this,
					&MainWindow::goClicked,			0 );
	prevCallback.connect( G_OBJECT( prevButton ),			"clicked",		this,	
					&MainWindow::prevClicked,		0 );	
	nextCallback.connect( G_OBJECT( nextButton ),			"clicked",		this,	
					&MainWindow::nextClicked,		0 );
	showHiddenCallback.connect( G_OBJECT( showHiddenItem ),	"activate",		this,	
					&MainWindow::showHiddenClicked,		0 );
	openTabCallback.connect( G_OBJECT( openTab ),			"activate",		this,	
					&MainWindow::openNewTab,		0 );
	openTabCallback.connect( G_OBJECT( open_Tab ),		"activate",		this,	
					&MainWindow::openNewTab,		0 );
	renameTabCallback.connect( G_OBJECT( renameItem ),	"activate",		this,	
					&MainWindow::renameTab,			0 );
	closeTabCallback.connect( G_OBJECT( closeTabItem ),		"activate",		this,	
					&MainWindow::closeTab,			0 );
	viewChangedCallback.connect( G_OBJECT( iconView ),		"activate",		this,	
					&MainWindow::changeViewMode,		0 );
	viewChangedCallback.connect( G_OBJECT( listView ),		"activate",		this,	
					&MainWindow::changeViewMode,		0 );
	viewChangedCallback.connect( G_OBJECT( imageView ),	"activate",		this,	
					&MainWindow::changeViewMode,		0 );
	openTerminalCallback.connect( G_OBJECT( newTerminalItem ),	"activate",		this,	
					&MainWindow::openNewTerminal,		0 );
	openSettingsCallback.connect( G_OBJECT( openSettings ),		"activate",		this,	
					&MainWindow::openSettingsClicked,	0 );
	openChangelogCallback.connect( G_OBJECT( changelogItem ),	"activate",		this,	
					&MainWindow::openChangelog,		0 );
	tabChangedCallback.connect_after( G_OBJECT( book ),		"switch-page",		this,	
					&MainWindow::tabChanged,		0 );
	showBottomBarCallback.connect( G_OBJECT( showBottomBar ),	"activate",		this,	
					&MainWindow::showBottomBarClicked,	NULL );
//NULL or 0, same for me as long as it works ;-)
}

//-------------------------------------------------------------------------
// destructor
MainWindow::~MainWindow()
{
	delete thumb;
}

//-------------------------------------------------------------------------
// addNewTab method
void MainWindow::addNewTab( const string& path )
{
	Tab *tab		=	new FileView( path );
	GtkWidget *window	=	tab->getWidget();

	string	t	=	"Browser "	+	Get::getString( id	+	1 );
	GtkWidget	*title	=	gtk_label_new( t.c_str() );
	
	GtkWidget *eventBox	=	gtk_event_box_new();
	
	gtk_event_box_set_above_child( GTK_EVENT_BOX( eventBox ),	true );
	gtk_event_box_set_visible_window( GTK_EVENT_BOX( eventBox ),	false );
	gtk_container_add( GTK_CONTAINER( eventBox ),	title );
	
	GtkWidget *box	=	gtk_hbox_new( false,	5 );
	gtk_box_pack_start( GTK_BOX( box ),	eventBox,	false,	true,	0 );
	
	gtk_notebook_append_page( book,	GTK_WIDGET( window ),	box );
	
	gtk_widget_show_all( eventBox );
	
	GdkDragAction	action		=	GdkDragAction( GDK_ACTION_COPY	|	GDK_ACTION_ASK );
	GdkModifierType	modifier	=	GdkModifierType( GDK_BUTTON1_MASK		|	GDK_BUTTON3_MASK );
		
	gtk_drag_dest_set( eventBox,		GTK_DEST_DEFAULT_ALL,	target,	3,	action );

	tabMenuCallback.connect( G_OBJECT( eventBox ),	"button-press-event",	this,	
				 &MainWindow::showTabMenu,		0 );
	dragReceiveCallback.connect( G_OBJECT( eventBox ),	"drag-data-received",	(FileView*)tab,
				     &FileView::dragReceived,	NULL );

	gtk_widget_show_all( window );
	tabList[ tab->getWidget() ]	=	tab;
	id++;
	this->showTabs();
}

//-------------------------------------------------------------------------
// updateStatusBar method
void MainWindow::updateStatusbar( string text )
{
	gtk_statusbar_push(	mainStatus,	0,	text.c_str() );
}

//-------------------------------------------------------------------------
// showTabMenu method
gboolean MainWindow::showTabMenu( GtkWidget *widget,	GdkEventButton *button,	gpointer data )
{	
	if( button->button != 3 )	// dummy test
		return false;

	gtk_menu_popup( GTK_MENU( tabMenu ),	0,	0,	0,	0,	3,	button->time );
	gtk_widget_show_all( tabMenu );
	return false;
}

//-------------------------------------------------------------------------
// renameTab method
void MainWindow::renameTab( GtkWidget *widget,	gpointer data )
{
	tab		=	getActiveTab();
	GtkWidget *w	=	tab->getWidget();

	string label	=	gtk_notebook_get_tab_label_text( book,	w );

	string newLabel	=	Dialog::createNewFuncDialog( "Rename tab",		"Rename",	label,	currentDir,	false );
	
	if( newLabel.length() > 0 )
		gtk_notebook_set_tab_label_text( book,	w,	newLabel.c_str() );
}

//-------------------------------------------------------------------------
// closeTab method
void MainWindow::closeTab( GtkWidget *widget,	gpointer data )
{
	int	page	=	gtk_notebook_get_current_page( book );
	gtk_notebook_remove_page( book,	page );
	this->showTabs();
}

//-------------------------------------------------------------------------
// changeViewMode method
void MainWindow::changeViewMode( GtkWidget *item,	gpointer data )
{
	if( !gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM( item ) ) )
		    return;
	
	tab	=	this->getActiveTab();
	fv	=	(FileView*)tab;
	
	fv->setView( GTK_RADIO_MENU_ITEM( item ) );
}

//-------------------------------------------------------------------------
// openNewTerminal method
void MainWindow::openNewTerminal(GtkWidget *widget, gpointer data)
{
	Tab *tab		=	new Terminal( currentDir,	false,	"" );
	
	GtkWidget *window	=	tab->getWidget();

	string t		=	"Terminal "	+	Get::getString( id + 1 );
	GtkWidget *title	=	gtk_label_new( t.c_str() );

      	tabMenuCallback.connect( G_OBJECT( title ),	"button-press-event",	this,	
				 &MainWindow::showTabMenu,		0 );

	gtk_notebook_append_page( book,	GTK_WIDGET( window ),	title );

	gtk_widget_show_all( window );
	tabList[ tab->getWidget() ]	=	tab;
	id++;
	this->showTabs();
}

//-------------------------------------------------------------------------
// showTabs method
void MainWindow::showTabs()
{
	int	ret	=	gtk_notebook_get_n_pages( book );
	bool	show	=	1;

	if( ret == 1 ) 
		show	=	false;
	
	gtk_notebook_set_show_tabs( book,	show );
}

//-------------------------------------------------------------------------
// openNewTerminalWithCommand method
void MainWindow::openNewTerminalWithCommand( string command )
{
	Tab *tab	=	new Terminal( currentDir,	true,	command );
	
	GtkWidget	*window	=	tab->getWidget();

	string	t	=	"Terminal "	+	Get::getString( id	+	1 );
	GtkWidget	*title	=	gtk_label_new( t.c_str() );

	tabMenuCallback.connect( G_OBJECT( title ),	"button-press-event",	this,	
				 &MainWindow::showTabMenu,		0 );

	int	page	=	gtk_notebook_append_page( book,	GTK_WIDGET( window ),	title );

	gtk_widget_show_all( window );
	tabList[ tab->getWidget() ]	=	tab;
	id++;
	this->showTabs();
	
	gtk_notebook_set_current_page( book,	page );
}

//-------------------------------------------------------------------------
// setColumnFeatures method
void MainWindow::setColumnFeatures( GtkTreeViewColumn *col,	int id )
{
	gtk_tree_view_column_set_resizable( col,	TRUE );
	gtk_tree_view_column_set_clickable( col,	TRUE );
	gtk_tree_view_column_set_sort_column_id( col,	id );
	gtk_tree_view_column_set_sort_indicator( col,	TRUE );
}

//-------------------------------------------------------------------------
// getSettings method
void MainWindow::getSettings()
{	
	showHidden	=	this->getBoolValue( "Values",	"showHidden" );
	if( showHidden == true )
		gtk_menu_item_activate( showHiddenItem );

	if( this->getBoolValue( "Values",	"showBottomToolbar" ) )
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( showBottomBar ),	true );
		
	bool	b	=	this->getBoolValue( "Values",	"viewMode" );
	if( b == true )
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( iconView ),	TRUE );
	else
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM( listView ),	TRUE );
		
	string	tabPos	=	getStringValue( "Values",	"tabPosition" );
	if( Util::compare( tabPos,	"left" ) ) 
		gtk_notebook_set_tab_pos( book,	GTK_POS_LEFT );
	else if( Util::compare( tabPos,	"right" ) )
		gtk_notebook_set_tab_pos( book,	GTK_POS_RIGHT );
	else if( Util::compare( tabPos,	"bottom" ) )
		gtk_notebook_set_tab_pos( book,	GTK_POS_BOTTOM );
	else 
		gtk_notebook_set_tab_pos( book,	GTK_POS_TOP );

	string toolbarStyle	=	getStringValue( "Values",	"toolbarStyle" );
	if( Util::compare( toolbarStyle,	"text" ) )
		gtk_toolbar_set_style( mainToolBar,	GTK_TOOLBAR_TEXT );
	else if( Util::compare( toolbarStyle,	"icons" ) )
		gtk_toolbar_set_style( mainToolBar,	GTK_TOOLBAR_ICONS );
	else
		gtk_toolbar_set_style( mainToolBar,	GTK_TOOLBAR_BOTH );

	themeName	=	this->getStringValue( "Values",	"iconTheme" );
	
	iconSize	=	this->getIntValue( "Values",	"iconSize" );
}

//-------------------------------------------------------------------------
// setSettings method
void MainWindow::setSettings( bool all )
{
	this->setValue( "Values",	"showHidden",		"",	0,	showHidden,	TYPE_BOOL );
	this->setValue( "Values",	"showBottomToolbar",	"",	0,	gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM( showBottomBar ) ),	TYPE_BOOL );
	if( all == true )
	{
		this->setValue( "Values",	"tabsOpened",	"",			1,	false,	TYPE_INT );
		this->setValue( "Values",	"iconSize",		"",		44,	false,	TYPE_INT );
		
		this->setValue( "Values",	"viewMode",	"",			0,	true,	TYPE_BOOL );
		
		this->setValue( "Values",	"tabPosition",	"left",			0,	false,	TYPE_STRING );
		this->setValue( "Values",	"loginShell",	"/bin/bash",		0,	false,	TYPE_STRING );
		this->setValue( "Values",	"iconTheme",	"GBrowser Default",	0,	false,	TYPE_STRING );
		this->setValue( "Values",	"defaultPath",	homeDir,		0,	false,	TYPE_STRING );
		this->setValue( "Values",	"toolbarStyle",	"both",			0,	false,	TYPE_STRING );
	}
}

//-------------------------------------------------------------------------
// getActiveTab method
Tab *MainWindow::getActiveTab()
{
        int page	=	gtk_notebook_get_current_page( book );
	GtkWidget *w	=	gtk_notebook_get_nth_page( book,        page );
	tabIter		=	tabList.find( w );

	if( tabIter == tabList.end() )
		return NULL;
	
	return tabIter->second;
}

//-------------------------------------------------------------------------
// setPath method
void MainWindow::setPath( const string& text )
{
	gtk_entry_set_text( pathEntry,	text.c_str() );
	gtk_entry_set_position( pathEntry,	text.size() );
}

//-------------------------------------------------------------------------
// setAllowNext method
void MainWindow::setAllowNext()
{
	if( allowNext == true )
	{
		allowNext	=	false;
		gtk_widget_set_sensitive( GTK_WIDGET( nextButton ),	false );
	}
}

//-------------------------------------------------------------------------
// setAllowPrev method
void MainWindow::setAllowPrev()
{
	if( allowPrev == false )
	{
		allowPrev	=	true;
		gtk_widget_set_sensitive( GTK_WIDGET( prevButton ),	true );
	}
}

//-------------------------------------------------------------------------
// setValue method
void MainWindow::setValue( const string& group,	const string& key,	const string& str,
							int i,	bool b,	int valueType )
{
	GKeyFile*	values	=	g_key_file_new();

	g_key_file_load_from_file( values,	valuesFile.c_str(),	G_KEY_FILE_NONE,	0 );
	
	if( valueType == TYPE_STRING ) 
		g_key_file_set_string( values,	group.c_str(),	key.c_str(),	str.c_str() );

	if( valueType == TYPE_BOOL )
		g_key_file_set_boolean( values,	group.c_str(),	key.c_str(),	b );

	if( valueType == TYPE_INT ) 
		g_key_file_set_integer( values,	group.c_str(),	key.c_str(),	i );
	
	ofstream	val( valuesFile.c_str() );
	char	*v	=	g_key_file_to_data( values,	0,	0 );
	val	<<	v;
	val.close();
	g_free( v );
	g_key_file_free( values );
}

//-------------------------------------------------------------------------
// getStringValue method
const char *MainWindow::getStringValue(const string& group,		const string& key)	const
{
	GKeyFile*	values	=	g_key_file_new();
	g_key_file_load_from_file( values,	valuesFile.c_str(),	G_KEY_FILE_NONE,	0 );

	if( !g_key_file_has_key( values,	group.c_str(),	key.c_str(),	NULL ) )
	{
		cout << "ERROR: GBrowser load config failed.\nKey: " << key << " not found from group " << group << "\n" << 
			"You should remove ~/.gbrowser for clean config files\n" << endl; 
		return "";
	}

	char	*d	=	g_key_file_get_string( values,	group.c_str(),	key.c_str(),	0);
	g_key_file_free( values );
	return d;
}

//-------------------------------------------------------------------------
// getBoolValue method
bool MainWindow::getBoolValue( const string& group,		const string& key )	const
{
	GKeyFile*	values	=	g_key_file_new();
	g_key_file_load_from_file( values,	valuesFile.c_str(),	G_KEY_FILE_NONE,	0 );

	if( !g_key_file_has_key( values,	group.c_str(),	key.c_str(),	NULL ) )
	{
		cout << "ERROR: GBrowser load config failed.\nKey: " << key << " not found from group " << group << "\n" << 
			"You should remove ~/.gbrowser for clean config files\n" << endl;
		return false;
	}

	bool	b	=	g_key_file_get_boolean( values,	group.c_str(),	key.c_str(),	0 );
	g_key_file_free( values );
	return b;
}

//-------------------------------------------------------------------------
// getIntValue method
int MainWindow::getIntValue( const string& group,	const string& key ) const
{
	GKeyFile*	values	=	g_key_file_new();
	g_key_file_load_from_file( values,	valuesFile.c_str(),	G_KEY_FILE_NONE,	0 );

	if( !g_key_file_has_key( values,	group.c_str(),	key.c_str(),	NULL ) )
	{
		cout << "ERROR: GBrowser load config failed.\nKey: " << key << " not found from group " << group << "\n" << 
			"You should remove ~/.gbrowser for clean config files\n" << endl;
		return -1;
	}

	int	i	=	g_key_file_get_integer( values,	group.c_str(),	key.c_str(),	0 );
	g_key_file_free( values );
	return i;
}

//-------------------------------------------------------------------------
// getGladeFile method
string MainWindow::getGladeFile( string file )	const
{
	string	dataPath	=	GLADEDIR"/"	+	file;

	if( !g_file_test( dataPath.c_str(),	G_FILE_TEST_EXISTS ) )
		dataPath	=	g_path_get_dirname( exec.c_str() )	+	(string)"/glade/"	+	file;

	if( !g_file_test( dataPath.c_str(),	G_FILE_TEST_EXISTS ) )
	{
		cout	<<	"WARNING!!!!! Couldn't find "	+ file	+	" file"	<<	endl;
		return	"";
	}
	return	dataPath;
}

//-------------------------------------------------------------------------
// getIcon method
GdkPixbuf	*MainWindow::getIcon( string path,	int size,	bool findByMime )	const
{
	const char	*mime	=	(char*)path.c_str();
	if( findByMime == false)
		mime	=	(char*)xdg_mime_get_mime_type_for_file( path.c_str() );
	
	string	icon	=	"mime-" + (string)mime;
	
	GdkPixbuf	*buf;
			
		if( ( g_file_test( path.c_str(),	G_FILE_TEST_IS_DIR ) ) )
			icon	=	"gnome-fs-directory";
		
		if( ( !gtk_icon_theme_has_icon( theme,	icon.c_str() ) )  &&
		    ( g_file_test( path.c_str(),  G_FILE_TEST_IS_DIR ) ) )
			icon	=	"mime-inode:directory";
		
		if( ( !gtk_icon_theme_has_icon( theme,	icon.c_str() ) ) )
		{
			string test	=	icon;
			int pos		=	test.find( "/", 0 );
			
			if( pos != string::npos )
				test[pos]	=	'-';
			
			while( 1 )
			{	
				if( Util::compare( test.substr( 0,	6 ),	"gnome-" ) )
				{
					string tmp	=	"gnome-" + icon;
					tmp[ pos ]	=	'-';

					if( ( gtk_icon_theme_has_icon( theme,	tmp.c_str() ) ) )
						icon	=	tmp;
					else
						icon	=	test;
					
					break;
				}
				
				if( ( gtk_icon_theme_has_icon( theme,	test.c_str() ) ) )
				{	
					icon	=	test;
					break;
				}
				
				test	=	test.substr( 0,	pos );
				
				if( ( gtk_icon_theme_has_icon( theme,	test.c_str() ) ) )
				{
					icon	=	test;
					break;
				}
				
				pos	+=	6;
				test.insert( 0,	"gnome-",	6 );
			}
		}

		if( (!gtk_icon_theme_has_icon( theme,	icon.c_str() ) ) )
		{
			string tmp	=	icon;
		
			if( ( !gtk_icon_theme_has_icon( theme,	tmp.c_str() ) ) && 
			  ( g_str_has_suffix( path.c_str(),	".mp3" ) ) ) 
				tmp	=	"gnome-mime-audio-mpeg";
		
			if( ( !gtk_icon_theme_has_icon( theme,	tmp.c_str() ) ) && 
		  	  ( g_str_has_suffix( path.c_str(),	".avi" ) ) ) 
				tmp	=	"mime-video";

			//for some reason following icons can't be found with some icon theme
			if( !gtk_icon_theme_has_icon( theme,	tmp.c_str() ) &&
			    g_str_has_suffix( path.c_str(),	".bz2" ) )
				tmp	=	"gnome-mime-application-x-bzip";

		        if( !gtk_icon_theme_has_icon( theme,	tmp.c_str() ) &&
			    g_str_has_suffix( path.c_str(),	".gz" ) )
				tmp	=	"gnome-mime-application-x-gzip";

		        if( !gtk_icon_theme_has_icon( theme,	tmp.c_str() ) &&
			    g_str_has_suffix( path.c_str(),	".tar" ) )
				tmp	=	"gnome-mime-application-x-tar";

			if( !gtk_icon_theme_has_icon( theme,	tmp.c_str() ) ) 
				tmp	=	"mime-text";
		
			if( !gtk_icon_theme_has_icon( theme,	tmp.c_str() ) ) 
				tmp	=	"gnome-mime-text";

			icon	=	tmp;

		}
			
	buf	=	gtk_icon_theme_load_icon( theme,	icon.c_str(),	size,
					GTK_ICON_LOOKUP_USE_BUILTIN,	0 );
			
	return buf;
}

//-------------------------------------------------------------------------
// getChangeLog method
string MainWindow::getChangelog()	const
{
	string	retVal	=	"";
	int		size		=	exec.length();
	string	s		=	exec.substr( 0,	size	-	8 );
	string	path		=	s	+	"ChangeLog";

	if( g_file_test( path.c_str(),	G_FILE_TEST_EXISTS ) ) 
		retVal	=	path;

	return retVal;
}

//-------------------------------------------------------------------------
// quitClicked method
void MainWindow::quitClicked(GtkWidget *widget, gpointer data)
{
	setSettings(false);
	
	for(tabIter = tabList.begin(); tabIter != tabList.end(); tabIter++) 
		delete tabIter->second;
	
	pasteData.clear();
	deleteData.clear();
	tabList.clear();
	
	gtk_main_quit();
}

//-------------------------------------------------------------------------
// deleteEvent method
gboolean MainWindow::deleteEvent(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	return false;
}

//-------------------------------------------------------------------------
// parentClicked method
void MainWindow::parentClicked(GtkWidget *widget, gpointer data)
{	
	tab	=	this->getActiveTab(); 
	fv	=	(FileView*)tab;
	
	string current	=	fv->getPath();
	if( Util::compare( current,	"/" ) ) 
		return;
	
	string::size_type pos	=	current.find_last_of( "/",	string::npos );
	
	string path;
	
	if( pos	!= 0 ) 
		path	=	current.substr( 0,	pos );
	else 
		path	=	current.substr( 0,	pos + 1 );
	
	fv->handleParent( path );
	
	if( !allowPrev )
		allowPrev	=	true;
	
	gtk_widget_set_sensitive( GTK_WIDGET( prevButton ),	allowPrev );
}

//-------------------------------------------------------------------------
// goClicked method
void MainWindow::goClicked( GtkWidget *widget,	gpointer data )
{
	string path		=	gtk_entry_get_text( pathEntry );
	string::size_type pos	=	path.find_last_of( "/" );
	
	if( g_str_has_suffix( path.c_str(),	"/" ) ) 
		path	=	path.substr( 0,	pos );

	if( !g_file_test( path.c_str(),	G_FILE_TEST_IS_DIR ) )
	{
		string text =	path + (string)" does not exist or it isn't a directory";
		this->updateStatusbar( text );
		return;
	}

	tab	=	getActiveTab();
	fv	=	(FileView*)tab;
	
	fv->clearTreeViews();
	fv->startScanning( path );

	if( allowNext )
	{
		allowNext	=	false;
		gtk_widget_set_sensitive( GTK_WIDGET( nextButton ),	false );
	}
	if( !allowPrev )
	{
		allowPrev	=	true;
		gtk_widget_set_sensitive( GTK_WIDGET( prevButton ),	true );
	}
}

//-------------------------------------------------------------------------
// prevClicked method
void MainWindow::prevClicked( GtkWidget *widget,	gpointer data )
{
	tab	=	getActiveTab();
	fv	=	(FileView*)tab;
	
	list<string> l = fv->getPrevList();
	
	if(l.begin() == l.end()) 
		return;
	
	list<string>::iterator it = l.begin();
	l.pop_front();

	fv->handlePrev((*it));

	if(!allowNext){
		allowNext = true;
		gtk_widget_set_sensitive(GTK_WIDGET(nextButton), true);
	}
	if(l.begin() == l.end()){
		allowPrev = false;
		gtk_widget_set_sensitive(GTK_WIDGET(prevButton), false);
	}
}

//-------------------------------------------------------------------------
// nextClicked method
void MainWindow::nextClicked(GtkWidget *widget, gpointer data)
{
	tab	=	getActiveTab();
	fv	=	(FileView*)tab;
	
	list<string> l = fv->getNextDirs();
	if(l.begin() == l.end()) 
		return;
	list<string>::iterator it = l.begin();

	fv->handleNext((*it));
	l.pop_front();

	if(l.begin() == l.end()){
		allowNext = false;
		gtk_widget_set_sensitive(GTK_WIDGET(nextButton), false);
	}
	if(!allowPrev){
		allowPrev = true;
		gtk_widget_set_sensitive(GTK_WIDGET(prevButton), true);
	}
}

//-------------------------------------------------------------------------
// showHiddenClicked method
void MainWindow::showHiddenClicked(GtkWidget *widget, gpointer data)
{
	if(showHidden) showHidden = false;
	else showHidden = true;

	tab	=	getActiveTab();
	fv	=	(FileView*)(tab);
	
	fv->setShowHidden(showHidden);
	fv->scanDirectory(fv->getPath());
}

//-------------------------------------------------------------------------
// tabChanged method
void MainWindow::tabChanged(GtkNotebook *notebook, GtkNotebookPage *p, int page, gpointer data)
{		
	tab	=	this->getActiveTab();
	tab->tabChanged();
}

//-------------------------------------------------------------------------
// openNewTab method
void MainWindow::openNewTab(GtkWidget *widget, gpointer data)
{
	tab	=	getActiveTab();
	
	addNewTab( g_get_home_dir() );
}

//-------------------------------------------------------------------------
// openSettingsClicked method
void MainWindow::openSettingsClicked(GtkWidget *widget, gpointer data)
{
	Settings::getInstance( this );
}

//-------------------------------------------------------------------------
// openChangelog method
void MainWindow::openChangelog(GtkWidget *widget, gpointer data)
{

        string	path	=	this->getChangelog();
	if( !g_file_test( path.c_str(),	G_FILE_TEST_EXISTS ) ) 
		return;
	
	ifstream	file( path.c_str() );

	stringstream	stream;
	
	stream	<<	file.rdbuf();
	
	string	content	=	stream.str();
	
	Dialog::dialogWithTextEntry( "ChangeLog",	content );
}

//-------------------------------------------------------------------------
// handleFile method
void MainWindow::handleFile( GtkListStore *store, string realName,	string fullPath,	
			     bool setFocus,	int type,	string currentDir )
{
	if( store	!=	Sidebar::getInstance()->boardStore )
	{
		if( !Util::compare( g_path_get_dirname( fullPath.c_str() ),	currentDir ) ) 
			return;
	}

	GdkPixbuf *buf	=	NULL;
	GtkTreeIter tree_iter;
	
	const char *mime	=	xdg_mime_get_mime_type_for_file( fullPath.c_str() );
	
	gtk_list_store_append( store,	&tree_iter );
	
	int size	=	iconSize;
	
	if( type == LIST_VIEW )
		size	=	16;
	
	if( ( mime != NULL ) && ( g_str_has_prefix( mime, "image" ) ) )	
		buf	=	thumb->getThumbnailFromDatabase( store,	tree_iter,	
								 fullPath,	size );
	if( buf == NULL )
		buf	=	this->getIcon( fullPath,	size,	false );
		
	const char *f	=	Get::getUtf8( realName );
	
	bool dir	=	g_file_test( fullPath.c_str(),	G_FILE_TEST_IS_DIR );
	
	gtk_list_store_set( store,	&tree_iter, 
				COLUMN_ICON,	buf, 
				COLUMN_NAME,	f,
				COLUMN_PATH,	fullPath.c_str(),
				COLUMN_IS_DIR,	dir,
				-1 );

	if( type	==	LIST_VIEW )
	{
		struct stat *statBuf	=	new struct stat;
		
		if( g_lstat( fullPath.c_str(),	statBuf ) != 0 )
			return;
		
		string accessTime	=	ctime( &statBuf->st_atime );
		accessTime.erase( accessTime.size() - 1,	accessTime.npos );
		
		string user	=	Get::getUser( statBuf->st_uid );
		string group	=	Get::getGroup( statBuf->st_gid );
		
		gtk_list_store_set( store,	&tree_iter,
				    COLUMN_OWNER,	user.c_str(),
				    COLUMN_GROUP,	group.c_str(),
				    COLUMN_ACCESS,	accessTime.c_str(),	-1 );

		if( !dir )
			gtk_list_store_set( store,	&tree_iter,	
					COLUMN_SIZE, Get::getSize( Get::getFileSize( fullPath ) ), 
					-1 );
	}
	
	if( setFocus	&&	store	!=	Sidebar::getInstance()->boardStore )
		fv->setFocusOnItem( NULL,	NULL,	tree_iter );
}

void MainWindow::showBottomBarClicked( GtkWidget *widget,	gpointer data )
{
	if( gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM( showBottomBar ) ) )
		gtk_widget_show_all( bottomBar );
	else
		gtk_widget_hide_all( bottomBar );
}

//-------------------------------------------------------------------------
// setTerminalMode method
void MainWindow::setTerminalMode()
{
	gtk_widget_hide_all( GTK_WIDGET( mainToolBar ) );

	gtk_widget_set_sensitive( GTK_WIDGET( showHiddenItem ),	false );
	gtk_widget_set_sensitive( GTK_WIDGET( iconView ),				false );
	gtk_widget_set_sensitive( GTK_WIDGET( listView ),				false );
	gtk_widget_set_sensitive( GTK_WIDGET( imageView ),			false );

	gtk_widget_hide_all( sidebarBook );
	gtk_paned_set_position( mainPaned,	0 );
}

//-------------------------------------------------------------------------
// setTerminalMode method
void MainWindow::setViewMode()
{
	gtk_widget_show_all( GTK_WIDGET( mainToolBar ) );

	gtk_widget_set_sensitive( GTK_WIDGET( showHiddenItem ),	true );
	gtk_widget_set_sensitive( GTK_WIDGET( iconView ),				true );
	gtk_widget_set_sensitive( GTK_WIDGET( listView ),				true );
	gtk_widget_set_sensitive( GTK_WIDGET( imageView ),			true );

	gtk_widget_show_all( sidebarBook );
	gtk_paned_set_position( mainPaned,	150 );

	fv      =       (FileView*)tab;
	this->setPath( fv->getPath().c_str() );	
}

//-------------------------------------------------------------------------
// updateCounters method
void MainWindow::updateCounters( int itemCount,	int hiddenCount )
{
	string stat	=	 Get::getString( itemCount ) + " Items";
	gtk_statusbar_push(	itemCountStatus,	0,	stat.c_str() );

	stat	=	Get::getString( hiddenCount ) + " Hidden";
	gtk_statusbar_push(	hiddenCountStatus,	0,	stat.c_str() );
}

//-------------------------------------------------------------------------
// iconThemeChanged method
void MainWindow::iconThemeChanged( string newTheme )
{
	if( Util::compare( newTheme,	"System Default" ) )
		theme	=	gtk_icon_theme_get_default();
	else
	{
		theme	=	gtk_icon_theme_new();
		gtk_icon_theme_set_custom_theme( theme,	newTheme.c_str() );
	}
	
	themeName	=	newTheme;
	
	for( tabIter = tabList.begin();	tabIter != tabList.end();	tabIter++ )
	{
		fv	=	(FileView*)tabIter->second;
		
		if( !fv )
			continue;
		
		GtkListStore *store	=	fv->getStore();
		
		GtkTreeIter it;
		gtk_tree_model_get_iter_first( GTK_TREE_MODEL( store ),	&it );
		
		while( 1 )
		{
			const char *mime,	*path;
			
			gtk_tree_model_get( GTK_TREE_MODEL( store ),	&it,	COLUMN_PATH,	&path,	-1 );
			
			mime	=	xdg_mime_get_mime_type_for_file( path );
			
			if( !g_str_has_prefix( mime, "image" ) )	
			{
				GdkPixbuf *buf	=	this->getIcon( path,	iconSize,	false );
			
				gtk_list_store_set( store,	&it,	COLUMN_ICON,	buf,	-1 );
			}
			
			gtk_tree_model_iter_next( GTK_TREE_MODEL( store ),	&it );
			
			if( !gtk_list_store_iter_is_valid( store,	&it ) )
				break;
		}
	}
}
