/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <string>
#include <iostream>
#include <list>
#include <gdk/gdk.h>
#include <fstream>
#include <pthread.h>
#include <map>
#include <vector>
#include <vte/vte.h>
#include <glib/gstdio.h>
#include <sstream>
#include <pwd.h>
#include <grp.h>

#define GETTEXT_PACKAGE "gtk20"
#include <glib/gi18n-lib.h>

#if ENABLE_EXIF
	#include <libexif/exif-ifd.h>
	#include <libexif/exif-data.h>
	#include <libexif/exif-utils.h>
	#include <libexif/exif-loader.h>
	#include <libexif/exif-content.h>
#endif

#include "../xdgmime/xdgmime.h"
#include "func.h"
#include "pasteFunc.h"
#include "fileview.h"
#include "terminal.h"
#include "settings.h"
#include "dialog.h"
#include "get.h"
#include "tab.h"
#include "thumbnail.h"
#include "util.h"
#include "dispatch.h"
#include "sidebar.h"

enum{
	TYPE_STRING,
	TYPE_INT,
	TYPE_BOOL
};

class MainWindow
{
public:
	~MainWindow();

	// set/get methods
	static MainWindow 	*getInstance( const char *path	=	NULL,	
									  const std::string& exe	=	"" );

	void	setColumnFeatures	( GtkTreeViewColumn *col,	int id );
	void	setSettings		( bool all );
	void	setPath			( const std::string& text );
	void	setAllowNext();
	void	setAllowPrev();
	void	setValue		( const std::string& group,	const std::string& key, 
					  const std::string& val,	int i,	bool b,	int valueType );
	
	inline	void	setIconSize	( int size ) 			{ iconSize = size; };
	inline	void	setPasteData	( std::list<std::string> list )	{ pasteData = list; };
	inline	void	setDeleteData	( std::list<std::string> list ) { deleteData = list; };
	inline	void	setIconThemeName( const std::string& name )	{ themeName = name; };
	
	inline	GtkWidget		*getMainBox()		const	{ return mainBox; };
	inline	GtkWidget		*getWindow()		const	{ return GTK_WIDGET( mainWindow ); };
	inline	GtkWidget		*getToolbar()		const	{ return GTK_WIDGET( mainToolBar ); };
	inline	GladeXML		*getXml()		const	{ return xml; };
	inline	GtkAccelGroup	*getAccelGroup()		const	{ return group; };
	inline	bool			getHidden()		const	{ return showHidden; };
	inline	std::string		getHomeDir()		const	{ return homeDir; };
	inline	GtkNotebook		*getBook()		const	{ return book; };
	inline	std::string		getPathToExecutable()	const	{ return exec; };
	inline	std::list<std::string>	getPasteData()		const	{ return pasteData; };
	inline	std::list<std::string>	getDeleteData() 	const	{ return deleteData; };

	void		getSettings();
	Tab		*getActiveTab();
	std::string	getChangelog()	const;
	bool		getBoolValue		( const std::string& group,		const std::string& key )	const;
	int		getIntValue		( const std::string& group,		const std::string& key )	const;	
	GdkPixbuf	*getIcon		( std::string path,	int size,	bool findByMime )	const;
	std::string	getGladeFile		( std::string file )	const;
	const char	*getStringValue		( const std::string& group,		const std::string& key )	const;

	// general methods
	void	updateStatusbar( std::string text );
	void	handleProgressStatus( void * );
	void	handleSettings();
	void	addNewTab( const std::string& path );
	void	showTabs();
	void	setTerminalMode();
	void	setViewMode();
	void	handleFile			( GtkListStore *store,	std::string name,	
							std::string path,	bool setFocus,	int type,	std::string dir );
	void	updateCounters		( int itemCount,	int hiddenCount );
	void iconThemeChanged		( std::string newTheme );
	
	//signals
	void	goClicked						( GtkWidget *widget,	gpointer data );
	void	showHiddenClicked				( GtkWidget *widget,	gpointer data );
	void	nextClicked					( GtkWidget *widget,	gpointer data );
	void	prevClicked					( GtkWidget *widget,	gpointer data );
	void	parentClicked					( GtkWidget *widget,	gpointer data );
	void	quitClicked					( GtkWidget *widget,	gpointer data );
	void	renameTab					( GtkWidget *widget,	gpointer data );
	void	closeTab						( GtkWidget *widget,	gpointer data );
	void	openNewTerminal				( GtkWidget *widget,	gpointer data );
	void	openSettingsClicked			( GtkWidget *widget,	gpointer data );
	gboolean	deleteEvent				( GtkWidget *widget,	GdkEvent *event,	gpointer data );
	void	openNewTab					( GtkWidget *widget,	gpointer data );
	void	tabChanged					( GtkNotebook *notebook,	GtkNotebookPage *page,
										int page_num,	gpointer data );
	gboolean	showTabMenu				( GtkWidget *widget,	GdkEventButton *button,	gpointer data );
	void	changeViewMode				( GtkWidget *item,		gpointer data );
	void	openChangelog				( GtkWidget *widget,	gpointer data );
	void	showBottomBarClicked			( GtkWidget *widget,	gpointer data );
	void	openNewTerminalWithCommand	( std::string command );

	GtkRadioMenuItem	*iconView,	*listView,	*imageView;
	
protected:
	// members
	Tab		*tab;
	Terminal	*terminal;
	FileView	*fv;
	Thumbnail	*thumb;
	
	bool	showHidden,	allowNext,	allowPrev;
	
	std::list<std::string>	pasteData,	deleteData;
	
	std::map<GtkWidget *, Tab *>	tabList;
	std::map<GtkWidget *, Tab *>::iterator	tabIter;

	GtkWidget	*mainBox,	*tabMenu,	*bottomBar,	*sidebarBook;
	GtkWindow	*mainWindow;
	GtkNotebook	*book;
	GtkMenuItem	*quitItem,	*showHiddenItem,	*openTab,	*open_Tab,	*renameItem, 
				*closeTabItem,	*newTerminalItem,	*openSettings,	*changelogItem,	*showBottomBar;
	GtkHBox		*hBox;
	GladeXML	*xml;
	GtkEntry		*pathEntry;
	GtkToolButton	*prevButton,	*nextButton,	*goButton,	*parentButton;
	GtkToolbar	*mainToolBar;
	GtkAccelGroup	*group;
	GtkStatusbar	*mainStatus,	*itemCountStatus,	*hiddenCountStatus;
	GtkPaned		*mainPaned;
	GtkIconTheme	*theme;

	GtkScrolledWindow	*dboardWindow;

	int	id,	iconSize;
	const char	*homeDir;

	std::string	exec,	valuesFile,	currentDir,	themeName;
	
	Func2<MainWindow, void, GtkWidget*>	quitCallback,			parentCallback,			goCallback,			
										prevCallback,			nextCallback,			showHiddenCallback,		
										openTabCallback,		renameTabCallback,		closeTabCallback,	
										viewChangedCallback,	openTerminalCallback,	openSettingsCallback,	
										openChangelogCallback,	parentCallbackAcc,		prevCallbackAcc,		
										nextCallbackAcc,	showBottomBarCallback;
	Func3<MainWindow, gboolean, GtkWidget *, GdkEventButton *>	tabMenuCallback;
	Func4<MainWindow, void, GtkNotebook *, GtkNotebookPage *, int>	tabChangedCallback;
	Func8<FileView, void, GtkWidget *, GdkDragContext *, int, int, GtkSelectionData *, guint, guint>	dragReceiveCallback;
	
private:
	MainWindow( const char *path,	const std::string& executable );

	static MainWindow *mainWin;
};

#else
class MainWindow;
#endif
