/***************************************************************************
 *   Copyright (C) 2005 by s4kk3   										   *
 *   s4kk33@gmail.com   												   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "settings.h"
#include "themedownloader.h"

using namespace std;

Settings *Settings::settings	=	NULL;
Settings *Settings::getInstance( MainWindow *mw )
{
	if( !settings )
		settings	=	new Settings( mw );
	return settings;
}

//-------------------------------------------------------------------------
// constructor
Settings::Settings( MainWindow *m )
{
	const string& dataPath	=	m->getGladeFile( "settings.glade" );
	xml	= 	glade_xml_new( dataPath.c_str(),	NULL,	NULL );

	//load widgets
	window		= 	glade_xml_get_widget( 			xml,	"settings" );
	
	okButton			=	GTK_BUTTON( glade_xml_get_widget( 	xml,	"okbutton1" ) );
	cancelButton		=	GTK_BUTTON( glade_xml_get_widget( 	xml,	"cancelbutton1" ) );
	applyButton		=	GTK_BUTTON( glade_xml_get_widget( 	xml,	"applybutton1" ) );
	downloadButton	=	GTK_BUTTON( glade_xml_get_widget(	xml,	"button1" ) );
	
	defPathEntry	=	GTK_ENTRY( 	glade_xml_get_widget( 	xml,	"path" ) );
	
	tabsOpened	=	GTK_SPIN_BUTTON( glade_xml_get_widget( 	xml,	"tabs" ) );
	
	view				=	GTK_TREE_VIEW( glade_xml_get_widget( 	xml,	"run_actions_view" ) );
	iconThemeView	=	GTK_TREE_VIEW( glade_xml_get_widget(	xml,	"icon_theme_view" ) );
	
	iconView	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"icon_view" ) );
	listView	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"list_view" ) );
	tabsLeft	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"tabsLeft" ) );
	tabsRight	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"tabsRight" ) );
	tabsBottom	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"tabsBottom" ) );
	tabsTop		=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"tabsTop" ) );
	styleText	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"radiobutton2" ) );
	styleIcons	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"radiobutton1" ) );
	styleBoth	=	GTK_RADIO_BUTTON( glade_xml_get_widget( xml,	"radiobutton3" ) );
	
	loginShellEntry	=	GTK_ENTRY( glade_xml_get_widget( 	xml,	"shellEntry" ) );

	iconOptionsBox	=	GTK_BOX( glade_xml_get_widget(		xml,	"hbox8" ) );
	
	mw	=	m;

	//load general settings
	const char *path	=	mw->getStringValue( "Values",	"defaultPath" );
	gtk_entry_set_text( defPathEntry,	path );
	
	gtk_spin_button_set_value(	tabsOpened,	double( mw->getIntValue( "Values",	"tabsOpened" ) ) );
	
	//create Icon theme treeview
	GtkTreeViewColumn *col;
	GtkCellRenderer *renderer;

	iconStore	=	gtk_list_store_new( 2,	G_TYPE_STRING,	G_TYPE_STRING );
	
	gtk_tree_view_set_model( iconThemeView,	GTK_TREE_MODEL( iconStore ) );
	
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Icon Theme" );
	gtk_tree_view_append_column( iconThemeView,	col );
	renderer	=	gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start( col,	renderer,	true );
	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	0 );

	 gtk_tree_view_set_headers_visible( iconThemeView,	false );
	
	//load interface settings
	bool b	=	mw->getBoolValue( "Values",	"viewMode" );
	
	if( b ) 
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( iconView ),	true);
	
	else 
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( listView ),	true);
	
	this->loadTabPosition();
	this->loadToolbarStyle();

	iconSizeEntry	=	GTK_COMBO_BOX( gtk_combo_box_new_text() );

	gtk_widget_set_size_request( GTK_WIDGET( iconSizeEntry ),	100,	-1 );

	GtkWidget *lab	=	gtk_label_new( "Icon Size" );
	gtk_box_pack_end( iconOptionsBox,	GTK_WIDGET( iconSizeEntry ),	false,	false,	5 );
	gtk_box_pack_end( iconOptionsBox,	lab,	false,	false,	5 );

	gtk_widget_show_all( GTK_WIDGET( iconOptionsBox ) );
	
	this->scanIconThemes();
	this->iconThemeSizes();
	
	int iconSize	=	mw->getIntValue( "Values",	"iconSize" );
	int pos		=	this->sizeEntryFindPos( Get::getString( iconSize ) );

	gtk_combo_box_set_active( iconSizeEntry,	pos );
	
	//load terminal settings
	const char	*shell	=	mw->getStringValue( "Values",	"loginShell" );
	gtk_entry_set_text( loginShellEntry,	shell );
	
	//create RunAction treeview
	store	=	gtk_tree_store_new( 4,	GDK_TYPE_PIXBUF,	G_TYPE_STRING,	
										G_TYPE_STRING,		G_TYPE_BOOLEAN );
	gtk_tree_view_set_model( view,	GTK_TREE_MODEL( store ) );
	
	//Icon
  	col	=	gtk_tree_view_column_new();
  	gtk_tree_view_column_set_title(		col,	"" );
  	gtk_tree_view_append_column( 	 	view,	col );
  	renderer	=	gtk_cell_renderer_pixbuf_new();
  	gtk_tree_view_column_pack_start(	col,	renderer,	TRUE );
  	gtk_tree_view_column_add_attribute( col,	renderer,	"pixbuf",	COL_ICO );
	
	//mime type
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( 	col,	"Mime type" );
  	gtk_tree_view_append_column( 		view,	col );
  	renderer	=	gtk_cell_renderer_text_new();
  	gtk_tree_view_column_pack_start( 	col,	renderer,	TRUE );
  	gtk_tree_view_column_add_attribute( col,	renderer,	"text",		COL_MIME );
	
	//run action
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Run Action" );
  	gtk_tree_view_append_column( view,	col );
  	renderer	=	gtk_cell_renderer_text_new();
  	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
  	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	COL_ACTION );
	g_object_set( renderer,	"editable",	TRUE,	NULL);	
	editFinishedCallback.connect( G_OBJECT( renderer ),	"edited",	this,	&Settings::editFinished,	NULL );
	
	//needs terminal?
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Needs Terminal" );
	gtk_tree_view_append_column( view,	col );
	renderer	=	gtk_cell_renderer_toggle_new();
	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
	gtk_tree_view_column_add_attribute( col,	renderer,	"active",	COL_TOGGLE );
	
	this->scanRunActions();
	
	cancelCallback.connect( 	G_OBJECT( cancelButton ),	"clicked",	this,	
				&Settings::cancelClicked,	NULL );
	okCallback.connect( 		G_OBJECT( okButton ),		"clicked",	this,	
				&Settings::okClicked,		NULL );
	okCallback.connect( 		G_OBJECT( applyButton ),	"clicked",	this,	
				&Settings::okClicked,		NULL );
	toggleCallback.connect( 	G_OBJECT( renderer ),		"toggled",	this,	
				&Settings::toggled,			NULL );
	downloadCallback.connect(	G_OBJECT( downloadButton ),	"clicked",	this,
				&Settings::downloadClicked,	NULL );
	destroyCallback.connect(	G_OBJECT( window ),		"destroy",	this,
				&Settings::onDestroyEvent,	NULL );
}

//-------------------------------------------------------------------------
// destructor
Settings::~Settings()
{
}

//-------------------------------------------------------------------------
// okClicked method
void Settings::okClicked( GtkWidget *widget,	gpointer data )
{
	mw->setValue( "Values",	"defaultPath",	gtk_entry_get_text( defPathEntry ),	
					0,	false,	TYPE_STRING );
	
	mw->setValue( "Values",	"tabsOpened",	"",	gtk_spin_button_get_value_as_int( tabsOpened ),	
					false,	TYPE_INT );
	
	mw->setValue( "Values",	"viewMode",	"",	0,	gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( iconView ) ), 
					TYPE_BOOL );
	
	const char*	home	=	g_get_home_dir();
	const string&	file	=	home	+	( string )"/.gbrowser/actions";
	
	GKeyFile*	actions	=	g_key_file_new();
	g_key_file_load_from_file( actions,	file.c_str(),	G_KEY_FILE_NONE,	NULL );
	
	gtk_tree_model_get_iter_first( GTK_TREE_MODEL( store ),	&iter );
	
	while( 1 )
	{
		GtkTreeIter subiter;

		const char *parentAction,	*parentMime,	*action,	*mime;

		gboolean parentToggled,	toggled;

		gtk_tree_model_get( GTK_TREE_MODEL( store ),	&iter, 
					COL_MIME,	&parentMime, 
					COL_ACTION,	&parentAction,
					COL_TOGGLE,	&parentToggled,
					-1 );

		if( parentAction )
		{
			string	act	=	parentAction;
			if( act[ act.size()	-	1]	==	' ' ) 
				act	=	act.substr( 0,	act.size()	-	1 );
		
			g_key_file_set_boolean( actions,	parentMime,	act.c_str(),	parentToggled );
		}

		if( gtk_tree_model_iter_children( GTK_TREE_MODEL( store ),	&subiter,	&iter ) )
			while( 1 )
			{
				gtk_tree_model_get( GTK_TREE_MODEL( store ),	&subiter, 
									COL_MIME,	&mime, 
									COL_ACTION,	&action, 
									COL_TOGGLE,	&toggled, 
									-1 );

				string	act	=	action;
				if( act[ act.size()	-	1]	==	' ' ) 
					act	=	act.substr( 0,	act.size()	-	1 );
		
				string fullMime	=	parentMime + (string)"/" + (string)mime;

				g_key_file_set_string( actions,	fullMime.c_str(),	"Default",	act.c_str() );
				g_key_file_set_boolean( actions,	fullMime.c_str(),	act.c_str(),	toggled );
		
				if( !gtk_tree_model_iter_next( GTK_TREE_MODEL( store ),	&subiter ) )
					break;
			}

		if( !gtk_tree_model_iter_next( GTK_TREE_MODEL( store ),	&iter ) )
			break;
	}
	
	ofstream val( file.c_str() );
	
	const char	*v	=	g_key_file_to_data( actions,	NULL,	NULL );
	val	<<	v;
	
	val.close();
	
	g_key_file_free( actions );
	
	this->saveTabPosition();
	this->saveToolbarStyle();

	mw->setValue( "Values",	"loginShell",	gtk_entry_get_text( loginShellEntry ), 
					0,	false,	TYPE_STRING );
	
	GtkTreeSelection *sel	=	gtk_tree_view_get_selection( iconThemeView );
	GtkTreeIter iter;
	const char *name;

	gtk_tree_selection_get_selected( sel,	NULL,	&iter );
	gtk_tree_model_get( GTK_TREE_MODEL( iconStore ),	&iter,	1,	&name,	-1 );

	mw->setValue( "Values",	"iconTheme",	name,	0,	false,	TYPE_STRING );
	
	int iconSize	=	(int)g_strtod( gtk_combo_box_get_active_text( iconSizeEntry ),	NULL );

	mw->setValue( "Values",	"iconSize",	"",	iconSize,	false,	TYPE_INT );
	mw->setIconSize( iconSize );
	
	mw->iconThemeChanged( name );

	if( widget	==	GTK_WIDGET( okButton ) )
		gtk_widget_destroy( window );
}

//-------------------------------------------------------------------------
// cancelClicked method
void Settings::cancelClicked( GtkWidget *widget,	gpointer data )
{
	gtk_widget_destroy( window );
}

//-------------------------------------------------------------------------
// editFinished method
void Settings::editFinished( GtkCellRenderer *renderer,	gchar *arg1,	gchar *arg2,	gpointer data )
{
	GtkTreeSelection	*selection	=	gtk_tree_view_get_selection( view );
	gtk_tree_selection_get_selected( selection,	NULL,	&iter );
	
	const string&	exe( arg2 );
	
	const string&	temp	=	exe.substr( 0,	exe.find( " ",	0 ) );
		
	if( Util::executableExists( temp ) )
		gtk_tree_store_set( store,	&iter,	COL_ACTION,	arg2,	-1 );
	
	else 
		Dialog::executableNotFound( temp );
}

//-------------------------------------------------------------------------
// toggled method
void Settings::toggled( GtkCellRendererToggle *cell,	gchar *path,	gpointer data )
{
	GtkTreeIter	iter;
  	gboolean	active;
	
  	gtk_tree_model_get_iter( GTK_TREE_MODEL( store ),	&iter, 
							gtk_tree_path_new_from_string( path ) );
  	
	gtk_tree_model_get( GTK_TREE_MODEL( store ),	&iter,	COL_TOGGLE,	&active,	-1 );
  	
	gtk_tree_store_set( store,	&iter,	COL_TOGGLE,	active?	false	:	true,	-1 );
}

//-------------------------------------------------------------------------
// scanIconThemes method
void Settings::scanIconThemes()
{
	gtk_list_store_clear( iconStore );
	
	GtkTreeIter iter;
	GtkTreeSelection *sel	=	gtk_tree_view_get_selection( iconThemeView );
	
	string currentTheme	=	mw->getStringValue( "Values",	"iconTheme" );
	
	gtk_list_store_append( iconStore,	&iter );
	gtk_list_store_set( iconStore,	&iter,	0,	"System Default",	1,	"System Default",	-1 );
	if( Util::compare( currentTheme,	"System Default" ) )
		gtk_tree_selection_select_iter( sel,	&iter );
	
	string path	=	"/usr/share/icons/";
	
	list<string> paths;
	paths.push_back( path );
	
	path	=	g_get_home_dir() + (string)"/.gbrowser/icons/";
	paths.push_back( path );

	for( list<string>::iterator it = paths.begin();	it != paths.end();	it++ )
	{
		path	=	(*it);
		
		GDir *dir	=	g_dir_open( path.c_str(),	0,	NULL );
		const char *name;

		while( ( name	=	g_dir_read_name( dir ) ) )
		{
			string tmp	=	path + name;
			if( g_file_test( tmp.c_str(),	G_FILE_TEST_IS_DIR ) )
			{
				string index	=	tmp + (string)"/index.theme";
				if( g_file_test( index.c_str(),	G_FILE_TEST_EXISTS ) )
				{
					GKeyFile	*indexFile	=	g_key_file_new();

					g_key_file_load_from_file( indexFile,	index.c_str(),	G_KEY_FILE_NONE,	NULL );
				
					const char *themeName	=	g_key_file_get_string( indexFile,	"Icon Theme",	"Name",	NULL );
					const char *themeDir	=	g_path_get_basename( tmp.c_str() );
				
					gtk_list_store_append( iconStore,	&iter );
					gtk_list_store_set( iconStore,	&iter,	0,	themeName,	1,	themeDir,	-1 );
					if( Util::compare( currentTheme,	themeDir ) )
						gtk_tree_selection_select_iter( sel,	&iter );
				}
			}
		}
	}
}

//-------------------------------------------------------------------------
// iconThemeSizes method
void Settings::iconThemeSizes()
{
	gtk_combo_box_append_text( iconSizeEntry,	"16" );
	gtk_combo_box_append_text( iconSizeEntry,	"24" );
	gtk_combo_box_append_text( iconSizeEntry,	"36" );
        gtk_combo_box_append_text( iconSizeEntry,	"48" );
	gtk_combo_box_append_text( iconSizeEntry,	"64" );
	gtk_combo_box_append_text( iconSizeEntry,	"72" );
	gtk_combo_box_append_text( iconSizeEntry,	"96" );
	gtk_combo_box_append_text( iconSizeEntry,	"128" );
}

//-------------------------------------------------------------------------
// sizeEntryFindPos method
int Settings::sizeEntryFindPos( const std::string& content )
{
	if( Util::compare( content,	"16" ) )
		return 0;
	else if( Util::compare( content,	"24" ) )
		return 1;
	else if( Util::compare( content,	"36" ) )
		return 2;
	else if( Util::compare( content,	"48" ) )
		return 3;
	else if( Util::compare( content,	"64" ) )
		return 4;
	else if( Util::compare( content,	"72" ) )
		return 5;
	else if( Util::compare( content,	"96" ) )
		return 6;
	else
		return 7;
}

//-------------------------------------------------------------------------
// downloadClicked method
void Settings::downloadClicked( GtkWidget *widget,	gpointer data )
{
	ThemeDownloader::getInstance();	
}

//-------------------------------------------------------------------------
// onDestroyEvent method
void Settings::onDestroyEvent( GtkWidget *widget,	gpointer data )
{
	delete settings;
	settings	=	NULL;
}

//-------------------------------------------------------------------------
// scanRunActions method
void Settings::scanRunActions()
{
	const char	*home	=	g_get_home_dir();
	const string&	file	=	home	+	( string )"/.gbrowser/actions";
	
	GKeyFile*	actions	=	g_key_file_new();
	g_key_file_load_from_file( actions,	file.c_str(),	G_KEY_FILE_NONE,	NULL );	
	
	char	**groups	=	g_key_file_get_groups( actions,	NULL );
	
	for( int i	=	0;	groups[ i ]	!=	NULL;	i++ )
	{
		if( groups[ i ]	==	NULL )
			break;

		const char *def	=	g_key_file_get_string( actions,	groups[i],	"Default",	NULL );
		bool terminal		=	g_key_file_get_boolean( actions,	groups[i],	def,	NULL );
		
		string	act	=	def;
		if( act[ act.size()	-	1 ]	==	' ' )
			act	=	act.substr( 0,	act.size()	-	1 );
		
		GtkTreeIter *parent	=	NULL;
		
		if( g_strrstr( groups[i],	"/" ) )
		{
			GtkTreeIter it;
			
			gtk_tree_model_get_iter_first( GTK_TREE_MODEL( store ),	&it );
			
			string parentMime	=	Get::getParentMime( groups[i] );
			
			bool force	=	false;
			
			while( 1 )
			{
				if( !gtk_tree_store_iter_is_valid( store,	&it ) || force )
				{
					gtk_tree_store_append( store,	&it,	NULL );
					gtk_tree_store_set( store,	&it,	
							COL_ICO,	mw->getIcon( parentMime,	16,	true ),
							COL_MIME,	parentMime.c_str(),
							COL_TOGGLE,	false,
							-1 );
						
					parent	=	gtk_tree_iter_copy( &it );
					break;
				}
				
				const char *tmp;
				
				gtk_tree_model_get( GTK_TREE_MODEL( store ),	&it,	COL_MIME,	&tmp,	-1 );
				
				if( Util::compare( tmp,	parentMime ) )
				{
					parent	=	gtk_tree_iter_copy( &it );
					break;
				}
				
				if( !gtk_tree_model_iter_next( GTK_TREE_MODEL( store ),	&it ) )
					force	=	true;
			}
		}
		
		string mime;
		if( !parent )
		{
			gtk_tree_store_append( store,	&iter,	NULL );
			mime	=	groups[ i ];
		}
		else
		{
			gtk_tree_store_append( store,	&iter,	parent );
			mime	=	Get::getSubMime( groups[ i ] );
		}
		
		gtk_tree_store_set( store,	&iter,
				    COL_ICO,	mw->getIcon( groups[ i ],	16,	true ),
				    COL_MIME,	mime.c_str(),
				    COL_ACTION,	def,
				    COL_TOGGLE,	terminal,
				    -1 );
		if( parent )
			gtk_tree_iter_free( parent );
	}
	
	g_strfreev( groups );
	g_key_file_free( actions );	
}

//-------------------------------------------------------------------------
// loadTabPosition method
void Settings::loadTabPosition()
{
	string tabPos	=	mw->getStringValue( "Values",	"tabPosition" );
	
	if( Util::compare( tabPos,	"left" ) ) 
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( tabsLeft ),	true );
	
	else if( Util::compare( tabPos,	"right" ) )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( tabsRight ),	true );
	
	else if( Util::compare( tabPos,	"bottom" ) )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( tabsBottom ),	true );
	
	else
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( tabsTop ),		true );
}

//-------------------------------------------------------------------------
// saveTabPosition method
void Settings::saveTabPosition()
{
	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( tabsLeft ) ) )
	{
		mw->setValue( "Values",	"tabPosition",	"left",	0,	false,	TYPE_STRING );
		gtk_notebook_set_tab_pos( mw->getBook(),	GTK_POS_LEFT );
	}
	
	else if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( tabsRight ) ) )
	{
		mw->setValue( "Values",	"tabPosition",	"right",	0,	false,	TYPE_STRING );
		gtk_notebook_set_tab_pos( mw->getBook(),	GTK_POS_RIGHT );
	}	
	
	else if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( tabsBottom ) ) )
	{
		mw->setValue( "Values",	"tabPosition",	"bottom",	0,	false,	TYPE_STRING );
		gtk_notebook_set_tab_pos( mw->getBook(),	GTK_POS_BOTTOM );
	}	
	
	else if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( tabsTop ) ) )
	{
		mw->setValue( "Values",	"tabPosition",	"top",	0,	false,	TYPE_STRING );
		gtk_notebook_set_tab_pos( mw->getBook(),	GTK_POS_TOP );
	}
}

//-------------------------------------------------------------------------
// loadToolbarStyle method
void Settings::loadToolbarStyle()
{
	string toolbarStyle	=	mw->getStringValue( "Values",	"toolbarStyle" );

	if( Util::compare( toolbarStyle,	"text" ) )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( styleText ),	true );
	else if( Util::compare( toolbarStyle,	"icons" ) )
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( styleIcons ),	true );
	else
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( styleBoth ),	true );
}

//-------------------------------------------------------------------------
// saveToolbarStyle method
void Settings::saveToolbarStyle()
{
	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( styleText ) ) )
	{
		mw->setValue( "Values",	"toolbarStyle",	"text",	0,	false,	TYPE_STRING );
		gtk_toolbar_set_style( GTK_TOOLBAR( mw->getToolbar() ),	GTK_TOOLBAR_TEXT );
	}
	else if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( styleIcons ) ) )
	{
		mw->setValue( "Values",	"toolbarStyle",	"icons",	0,	false,	TYPE_STRING );
		gtk_toolbar_set_style( GTK_TOOLBAR( mw->getToolbar() ),	GTK_TOOLBAR_ICONS );
	}
	else
	{
		mw->setValue( "Values",	"toolbarStyle",	"both",	0,	false,	TYPE_STRING );
		gtk_toolbar_set_style( GTK_TOOLBAR( mw->getToolbar() ),	GTK_TOOLBAR_BOTH );
	}
}
