/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#if ENABLE_ICONVIEW
	#include "gtkiconview.h"
#else
	#include <gtk/gtkiconview.h>
#endif

#include "sidebar.h"

using namespace std;

Sidebar *Sidebar::bar	=	NULL;
Sidebar *Sidebar::getInstance()
{
	if( !bar )
		bar	=	new Sidebar();
	return bar;
}

gboolean enterTimeout( void * )
{
	Sidebar::getInstance()->progressBarPush();	
	return true;
}

//-------------------------------------------------------------------------
// constructor
Sidebar::Sidebar()
{
	bar	=	this;

	GladeXML *xml	=	MainWindow::getInstance()->getXml();

	treeWindow		=	glade_xml_get_widget( xml,	"treeviewwindow" );
	dragWindow		=	glade_xml_get_widget( xml,	"dragboardwindow" );
	bookmarkWindow	=	glade_xml_get_widget( xml,	"bookmarkwindow" );
	
	treeViewBox		=	GTK_BOX( glade_xml_get_widget( xml,	"treeviewbox" ) );
	
	pBar				=	GTK_PROGRESS_BAR( glade_xml_get_widget( xml,	"progressbar1" ) );
	
	addBookButton		=	GTK_BUTTON( glade_xml_get_widget( xml,	"button5" ) );
	removeBookButton		=	GTK_BUTTON( glade_xml_get_widget( xml,	"button4" ) );
	
	dragEvent		=	GTK_EVENT_BOX( glade_xml_get_widget( xml,	"eventbox2" ) );

	directory	=	MainWindow::getInstance()->getIcon( "/",	12,	false );
	
	this->createTreeView();
	this->createDragBoard();
	this->createBookmarks();

	this->loadBookmarks();
	
	calls	=	0;

	rowExpandedCallback.connect( G_OBJECT( treeView ),				"row-expanded",		this,
					&Sidebar::rowExpanded,				NULL );
	treeViewActivatedCallback.connect( G_OBJECT( treeView ),			"row-activated",		this,
					&Sidebar::treeViewActivated,			NULL );
        getDataCallback.connect( G_OBJECT( boardView ),				"drag-data-get",		this,	
					&Sidebar::getDragData,				NULL );
	dragReceiveCallback.connect( G_OBJECT( dragWindow ),			"drag_data_received",	this,	
					&Sidebar::dragReceived,				NULL );
	dragReceiveCallback.connect( G_OBJECT( dragEvent ),			"drag_data_received",	this,	
				     &Sidebar::dragReceived,				NULL );
	bookViewActivatedCallback.connect( G_OBJECT( bookmarkView ),	"row-activated",		this,
					&Sidebar::bookViewActivated,		NULL );
	addBookmarkCallback.connect( G_OBJECT( addBookButton ),		"clicked",				this,
					&Sidebar::addBookmarkClicked,		NULL );
	removeBookmarkCallback.connect( G_OBJECT( removeBookButton ),	"clicked",				this,
				       &Sidebar::removeBookmarkClicked,	NULL );
	
	pathList.push_back( "/" );
	
	calls++;
	
	typedef Dp3<Sidebar, void, string, GtkTreeRowReference*, int> dp3;
	dp3 *func	=	new dp3( this,	&Sidebar::treeStoreScan,	"/",	NULL,	0 );

	Dispatch::getInstance()->addToQueue( func );
	
	id	=	g_timeout_add( 100,	&enterTimeout,	NULL );
}

//-------------------------------------------------------------------------
// destructor
Sidebar::~Sidebar()
{
}

//-------------------------------------------------------------------------
// createTreeView method
void Sidebar::createTreeView()
{
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *ren;
	
	treeStore	=	gtk_tree_store_new( 3,	GDK_TYPE_PIXBUF,	G_TYPE_STRING,	G_TYPE_STRING );
	treeView 	=	GTK_TREE_VIEW( gtk_tree_view_new_with_model( GTK_TREE_MODEL( treeStore ) ) );
	
	gtk_tree_view_set_headers_visible( treeView,	false );

  	col	=	gtk_tree_view_column_new();
  	gtk_tree_view_column_set_title( col,	"" );
  	gtk_tree_view_append_column( GTK_TREE_VIEW( treeView ),	col );
  	
	ren	=	gtk_cell_renderer_pixbuf_new();
  	gtk_tree_view_column_pack_start( col,	ren,	true );
  	gtk_tree_view_column_add_attribute( col,	ren,	"pixbuf",	0 );

        col	=	gtk_tree_view_column_new();
  	gtk_tree_view_column_set_title( col,	"" );
  	gtk_tree_view_append_column( GTK_TREE_VIEW( treeView ),	col );

  	ren	=	gtk_cell_renderer_text_new();
  	gtk_tree_view_column_pack_start( col,	ren,	TRUE );
  	gtk_tree_view_column_add_attribute( col,	ren,	"text",	1 );

	gtk_container_add( GTK_CONTAINER( treeWindow ),	GTK_WIDGET( treeView ) );

	gtk_tree_sortable_set_default_sort_func( GTK_TREE_SORTABLE( treeStore ),	Util::sortFuncOnlyDirs,	NULL,	NULL );
    	gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE( treeStore ), 
					      GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
					      GTK_SORT_ASCENDING );

	gtk_widget_show_all( treeWindow );
}

//-------------------------------------------------------------------------
// createDragBoard method
void Sidebar::createDragBoard()
{
	boardStore	=	gtk_list_store_new( 5,	GDK_TYPE_PIXBUF,	
							G_TYPE_STRING,	
							G_TYPE_STRING,	
							G_TYPE_STRING,	
							G_TYPE_BOOLEAN );

	boardView 	=	GTK_ICON_VIEW( gtk_icon_view_new_with_model( GTK_TREE_MODEL( boardStore ) ) );

	gtk_icon_view_set_selection_mode( GTK_ICON_VIEW( boardView ),	GTK_SELECTION_MULTIPLE );
	
	gtk_icon_view_set_text_column( GTK_ICON_VIEW( boardView ),	COLUMN_NAME );
	gtk_icon_view_set_pixbuf_column( GTK_ICON_VIEW( boardView ),	COLUMN_ICON );
	
	gtk_icon_view_set_item_width( GTK_ICON_VIEW( boardView ),	100 );
	
	gtk_container_add( GTK_CONTAINER( dragWindow ),	GTK_WIDGET( boardView ) );
	
	gtk_tree_sortable_set_default_sort_func( GTK_TREE_SORTABLE( boardStore ),	Util::sortFunc,	NULL,	NULL );
    	gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE( boardStore ), 
						GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
						GTK_SORT_ASCENDING );

	GdkModifierType	modifier		=	GdkModifierType( GDK_BUTTON1_MASK		|	GDK_BUTTON3_MASK );
	GdkDragAction	action		=	GdkDragAction( GDK_ACTION_COPY		|	GDK_ACTION_ASK );
	
	gtk_drag_dest_set( GTK_WIDGET( dragWindow ),	GTK_DEST_DEFAULT_ALL,	target,	3,	action );
	gtk_drag_dest_set( GTK_WIDGET( dragEvent ),	GTK_DEST_DEFAULT_ALL,	target,	3,	action );
	gtk_icon_view_enable_model_drag_source( boardView,		modifier,	target,	3,	action );
	
	gtk_widget_show_all( GTK_WIDGET( dragWindow ) );
}

//-------------------------------------------------------------------------
// createBookmarks method
void Sidebar::createBookmarks()
{
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *ren;
	
	bookmarkStore	=	gtk_list_store_new( 2,	G_TYPE_STRING,	G_TYPE_STRING );
	bookmarkView 	=	GTK_TREE_VIEW( gtk_tree_view_new_with_model( GTK_TREE_MODEL( bookmarkStore ) ) );
	
	gtk_tree_view_set_headers_visible( bookmarkView,	false );

	for( int i = 0;	i != 2;	i++ )
	{
        	col	=	gtk_tree_view_column_new();
  		gtk_tree_view_column_set_title( col,	"" );
  		gtk_tree_view_append_column( GTK_TREE_VIEW( bookmarkView ),	col );

  		ren	=	gtk_cell_renderer_text_new();
  		gtk_tree_view_column_pack_start( col,	ren,	true );
  		gtk_tree_view_column_add_attribute( col,	ren,	"text",	i );
	}

	gtk_container_add( GTK_CONTAINER( bookmarkWindow ),	GTK_WIDGET( bookmarkView ) );

	gtk_tree_sortable_set_default_sort_func( GTK_TREE_SORTABLE( bookmarkStore ),	Util::sortFuncOnlyDirs,	NULL,	NULL );
    	gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE( bookmarkStore ), 
					      GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
					      GTK_SORT_ASCENDING );

	gtk_widget_show_all( bookmarkWindow );	
}

//-------------------------------------------------------------------------
// treeStoreScan method
void Sidebar::treeStoreScan( string path,	GtkTreeRowReference *ref,	int depth )
{
	GtkTreeIter iter,	*parent;

	GDir *dir	=	g_dir_open( path.c_str(),	0,	NULL );

	if( !dir )
	{
		calls--;
		if( calls == 0 )
			gtk_widget_hide( GTK_WIDGET( pBar ) );
		return;
	}
	
	if( ref != NULL )
	{
		GtkTreePath *treePath	=	gtk_tree_row_reference_get_path( ref );

		gtk_tree_model_get_iter( GTK_TREE_MODEL( treeStore ),	&iter,	treePath );
		parent	=	gtk_tree_iter_copy( &iter );
		
		gtk_tree_path_free( treePath );
	}
	
	else
	{
		parent	=	NULL;
		path.erase( 0 );
	}

	const char *name	=	g_dir_read_name( dir );	

	while( name != NULL )
	{
        	string fullPath	=	path + (string)"/" + (string)name;

		if( !g_file_test( fullPath.c_str(),	G_FILE_TEST_IS_DIR )	||
		    g_str_has_prefix( name,	"." )	||
		    find( pathList.begin(),	pathList.end(),		fullPath ) != pathList.end() )
		{
			name	=	g_dir_read_name( dir );
			continue;	
		}

		gtk_tree_store_append( treeStore,		&iter,	parent );

		gtk_tree_store_set( treeStore,	&iter,	0,	directory,
							1,	name,
							2,	fullPath.c_str(),	-1 );

		pathList.push_front( fullPath );

		if( Util::checkReadPermissions( fullPath ) &&
		    depth < 1 )
		{
             		GtkTreePath *subPath			=	gtk_tree_model_get_path( GTK_TREE_MODEL( treeStore ),	&iter );
			GtkTreeRowReference *subRef	=	gtk_tree_row_reference_new( GTK_TREE_MODEL( treeStore ),	subPath );

			calls++;
				
			typedef Dp3<Sidebar, void, string, GtkTreeRowReference*, int> dp3;
			dp3 *func	=	new dp3( this,	&Sidebar::treeStoreScan,	fullPath,	subRef,	depth + 1 );

			Dispatch::getInstance()->addToQueue( func );

			gtk_tree_path_free( subPath );
		}

		name	=	g_dir_read_name( dir );	
	}
	
	calls--;
	
	if( calls == 0 )
		gtk_widget_hide( GTK_WIDGET( pBar ) );
	
	g_dir_close( dir );

	gtk_tree_row_reference_free( ref );
}

//-------------------------------------------------------------------------
// rowExpanded method
void Sidebar::rowExpanded( GtkTreeView *view,	GtkTreeIter *it,		GtkTreePath *p,	gpointer data )
{
	gtk_widget_show( GTK_WIDGET( pBar ) );
	
	GtkTreeIter subIter,	iter;

	gtk_tree_model_get_iter( GTK_TREE_MODEL( treeStore ),	&iter,	p );

	gtk_tree_model_iter_children( GTK_TREE_MODEL( treeStore ),	&subIter,	&iter );

	while( 1 )
	{                
		const char *fullPath;
		gtk_tree_model_get( GTK_TREE_MODEL( treeStore ),	&subIter,	2,	&fullPath,	-1 );

		GtkTreePath *subPath	=	gtk_tree_model_get_path( GTK_TREE_MODEL( treeStore ),	&subIter );
		GtkTreeRowReference *subRef	=	gtk_tree_row_reference_new( GTK_TREE_MODEL( treeStore ),	subPath );
		
		calls++;
		
		typedef Dp3<Sidebar, void, string, GtkTreeRowReference*, int> dp3;
		dp3 *func	=	new dp3( this,	&Sidebar::treeStoreScan,	fullPath,	subRef,	0 );

		Dispatch::getInstance()->addToQueue( func );

		gtk_tree_path_free( subPath );

		if( !gtk_tree_model_iter_next( GTK_TREE_MODEL( treeStore ),	&subIter ) )
			break;
	}
}

//-------------------------------------------------------------------------
// treeViewActivated method
void Sidebar::treeViewActivated( GtkTreeView *view,	GtkTreePath *p,	GtkTreeViewColumn *col,	gpointer data )
{
	GtkTreeIter iter;
	const char *path;

	gtk_tree_model_get_iter( GTK_TREE_MODEL( treeStore ),	&iter,	p );
	gtk_tree_model_get( GTK_TREE_MODEL( treeStore ),	&iter,	2,	&path,	-1 );

	Tab 		*tab	=	MainWindow::getInstance()->getActiveTab();
	FileView 	*fv	=	(FileView*)tab;
	
	typedef Dp1<FileView, void, string> dp1;
	dp1 *func	=	new dp1( fv,	&FileView::startScanning,	path );
	Dispatch::getInstance()->addToQueue( func );

	gtk_tree_view_expand_row( treeView,	p,	false );
}

//-------------------------------------------------------------------------
// treeViewSetPath method
void Sidebar::treeViewSetPath( string path )
{
	GtkTreeIter iter;

	gtk_tree_view_collapse_all( treeView );

	if( Util::compare( path,	"/" ) )
		return;

	gtk_tree_model_get_iter_first( GTK_TREE_MODEL( treeStore ),	&iter );

	while( 1 )
	{
        	const char *subPath;

		gtk_tree_model_get( GTK_TREE_MODEL( treeStore ),	&iter,	2,	&subPath,	-1 );

		string::size_type pos	=	path.find( "/",	strlen( subPath ) );
		string tmp	=	path.substr( 0,	pos );

		if( Util::compare( subPath,	path ) )
			break;

		else if( Util::compare( tmp,	subPath ) )
		{
			GtkTreeIter child;
			gtk_tree_model_iter_children( GTK_TREE_MODEL( treeStore ),	&child,	&iter );

			iter	=	child;
		}

		gtk_tree_model_iter_next( GTK_TREE_MODEL( treeStore ),	&iter );

		if( !gtk_tree_store_iter_is_valid( treeStore,	&iter ) )
			break;
	}

	GtkTreePath *treePath		=	gtk_tree_model_get_path( GTK_TREE_MODEL( treeStore ),	&iter );
	GtkTreeSelection *selection	=	gtk_tree_view_get_selection( treeView );

	gtk_tree_selection_unselect_all( selection );

	gtk_tree_view_expand_to_path( treeView, treePath );
	gtk_tree_selection_select_path( selection,	treePath );

	gtk_tree_view_scroll_to_cell( treeView,	treePath,	gtk_tree_view_get_column( treeView,	1 ),	
					false,
					0,
					0 );
}

//-------------------------------------------------------------------------
// dragReceived method
void Sidebar::dragReceived( GtkWidget *widget, 
							 GdkDragContext *context, 
							 int x, 
							 int y,
							 GtkSelectionData *data,
							 guint info,
							 guint time, 
							 gpointer user_data )		
{
	string path,	paths	=	( char * )data->data;
	string::size_type pos	=	0;

	paths	+=	"\n";
	
	GtkTreeIter iter;

	while( 1 )
	{
		pos	=	paths.find( "\n",	0 );
		
		if( pos	==	string::npos ) 
			break;
		
		path	=	paths.substr( 0,	pos );

		MainWindow::getInstance()->handleFile( boardStore,	g_path_get_basename( path.c_str() ),	
							path,	false,	ICON_VIEW,	"" );

		paths.erase( 0,	pos	+	1 );
	}
}

//-------------------------------------------------------------------------
// getDragData method
void Sidebar::getDragData( GtkWidget *widget,
				GdkDragContext *context,
				GtkSelectionData *data,
				guint info,
				guint time,
				gpointer user_data )
{	
	GtkTreeIter i;
	const char *path;
	list<string> dndData;
	
	GList *selections;
	
	selections	=	gtk_icon_view_get_selected_items( GTK_ICON_VIEW( boardView ) );
	
	if( selections	==	NULL ) 
		return;
	
	guint len	=	g_list_length( selections );
	char *paths[ len ];
	int it	=	0;
	
	GList *glist	=	g_list_first( selections );
	
	while( 1 )
	{	
		gtk_tree_model_get_iter( GTK_TREE_MODEL( boardStore ),	&i, 
								( GtkTreePath* )glist->data );
		gtk_tree_model_get( GTK_TREE_MODEL( boardStore ),	&i,
							COLUMN_PATH,	&path,	-1 );
		
		dndData.push_back( (char*)path );
		
		paths[ it ]	=	( char* )path;
		it++;
		
		glist	=	g_list_next( glist );
		
		if( !glist ) 
			break;
	}
	
	paths[ it ]	=	NULL;
	
	char *content	=	g_strjoinv( "\n",	paths );
	
    	gtk_selection_data_set( data,	data->target,	8, ( guchar* )content,	strlen( content ) );

	typedef Dp2< FileView, void, GtkListStore*, list<string> > dp2;
	dp2 *func	=	new dp2( (FileView*)MainWindow::getInstance()->getActiveTab(),	&FileView::removeEntries,	
					 boardStore,	dndData );
	
	Dispatch::getInstance()->addToQueue( func );
	
	g_list_free( glist );
	g_list_free( selections );
}

//-------------------------------------------------------------------------
// loadBookmarks method
void Sidebar::loadBookmarks()
{
	GKeyFile *file	=	g_key_file_new();
	string config	=	g_get_home_dir() + (string)"/.gbrowser/bookmarks";

	g_key_file_load_from_file( file,	config.c_str(),	G_KEY_FILE_NONE,	NULL );
	
	char **bookList	=	g_key_file_get_groups( file,	NULL );

	for( int it = 0;	bookList[it] != NULL;	it++ )
	{
		string name	=	bookList[it];
		string path	=	g_key_file_get_string( file,	name.c_str(),	"Path",	NULL );

		GtkTreeIter iter;

		gtk_list_store_append( bookmarkStore,	&iter );
		gtk_list_store_set( bookmarkStore,	&iter,	0,	name.c_str(),	1,	path.c_str(),	-1 );
	}

	g_key_file_free( file );
	g_strfreev( bookList );	
}

//-------------------------------------------------------------------------
// addBookmark method
void Sidebar::addBookmark( std::string path )
{
	string name	=	g_path_get_basename( path.c_str() );
	string config	=	g_get_home_dir() + (string)"/.gbrowser/bookmarks";
	GKeyFile *file	=	g_key_file_new();

	g_key_file_load_from_file( file,	config.c_str(),	G_KEY_FILE_NONE,	NULL );

	g_key_file_set_string( file,	name.c_str(),	"Path",	path.c_str() );

	GtkTreeIter iter;

	gtk_list_store_append( bookmarkStore,	&iter );
	gtk_list_store_set( bookmarkStore,	&iter,	0,	name.c_str(),	1,	path.c_str(),	-1 );
	
	ofstream	val( config.c_str() );
	char	*v	=	g_key_file_to_data( file, 0,	0 );
	val	<<	v;
	val.close();

	g_free( v );
	g_key_file_free( file );
}

//-------------------------------------------------------------------------
// bookViewActivated method
void Sidebar::bookViewActivated( GtkTreeView *view,	GtkTreePath *p,	GtkTreeViewColumn *col,	gpointer data )
{
	GtkTreeIter iter;
	const char *path;

	gtk_tree_model_get_iter( GTK_TREE_MODEL( bookmarkStore ),	&iter,	p );
	gtk_tree_model_get( GTK_TREE_MODEL( bookmarkStore ),	&iter,	1,	&path,	-1 );

	FileView *fv	=	(FileView*)MainWindow::getInstance()->getActiveTab();
	fv->startScanning( path );
}

void Sidebar::progressBarPush()	const
{
	gtk_progress_bar_pulse( pBar );
}

void Sidebar::addBookmarkClicked( GtkWidget *widget,		gpointer data )
{
	FileView *fv	=	(FileView*)MainWindow::getInstance()->getActiveTab();
	
	this->addBookmark( fv->getPath() );
}

void Sidebar::removeBookmarkClicked( GtkWidget *widget,	gpointer data )
{
	GtkTreeSelection *sel	=	gtk_tree_view_get_selection( bookmarkView );
	GtkTreeIter iter;
	
	gtk_tree_selection_get_selected( sel,	NULL,	&iter );
	
	const char *name;
	
	gtk_tree_model_get( GTK_TREE_MODEL( bookmarkStore ),	&iter,	0,	&name,	-1 );
	
	string config	=	g_get_home_dir() + (string)"/.gbrowser/bookmarks";
	GKeyFile *file	=	g_key_file_new();
	
	g_key_file_load_from_file( file,	config.c_str(),	G_KEY_FILE_NONE,	NULL );
	g_key_file_remove_group( file,	name,	NULL );
	
	gtk_list_store_remove( bookmarkStore,	&iter );
	
	ofstream	val( config.c_str() );
	char	*v	=	g_key_file_to_data( file, 0,	0 );
	val	<<	v;
	val.close();

	g_free( v );
	g_key_file_free( file );
}

