/***************************************************************************
 *   Copyright (C) 2005 by s4kk3                                           *
 *   s4kk33@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "themedownloader.h"
#include <curl/curl.h>

using namespace std;

ThemeDownloader *ThemeDownloader::themeDL	=	NULL;
ThemeDownloader *ThemeDownloader::getInstance()
{
	if( !themeDL )
		themeDL	=	new ThemeDownloader();
		
	return themeDL;
}

ThemeDownloader::ThemeDownloader()
{
	themeDL	=	this;	
	
	string dataPath	=	MainWindow::getInstance()->getGladeFile( "download_dialog.glade" );
	
	GladeXML *xml	=	glade_xml_new( dataPath.c_str(),	0,	0 );
	
	window			=	glade_xml_get_widget( xml,	"window1" );
	view				=	glade_xml_get_widget( xml,	"scrolledwindow" );	
	
	preview			=	GTK_IMAGE( glade_xml_get_widget( xml,	"image2" ) );
	
	themeName		=	GTK_LABEL( glade_xml_get_widget( xml,	"iconthemename" ) );
	authorLabel		=	GTK_LABEL( glade_xml_get_widget( xml,	"authorlabel" ) );
	descriptionLabel	=	GTK_LABEL( glade_xml_get_widget( xml,	"descriptionlabel" ) );
	urlLabel			=	GTK_LABEL( glade_xml_get_widget( xml,	"urlLabel" ) );
	emailLabel		=	GTK_LABEL( glade_xml_get_widget( xml,	"emailLabel" ) );
	
	pBar				=	GTK_PROGRESS_BAR( glade_xml_get_widget( xml,	"progressbar1" ) );
	
	closeButton		=	GTK_BUTTON( glade_xml_get_widget( xml,	"button1" ) );
	downloadButton	=	GTK_BUTTON( glade_xml_get_widget( xml,	"button2" ) );
	
	this->createThemeView();
	
	GtkTreeSelection *sel	=	gtk_tree_view_get_selection( themeView );
	
	selectionChangedCallback.connect( G_OBJECT( sel ),		"changed",	this,
					  &ThemeDownloader::themeSelectionChanged,	NULL );
	downloadCallback.connect( G_OBJECT( downloadButton ),	"clicked",		this,
				 	 &ThemeDownloader::downloadClicked,			NULL );
	closeCallback.connect( G_OBJECT( closeButton ),			"clicked",		this,
			      	 	&ThemeDownloader::closeClicked,				NULL );
	destroyCallback.connect( G_OBJECT( window ),			"destroy",		this,
				 	&ThemeDownloader::onDestroyEvent,			NULL );
	
	DownloadData *data	=	new DownloadData;
	
	data->url		=	"http://s4kk3.servebeer.com/icon_theme.index";
	data->name	=	"icon_theme.index";
	
	pthread_create( &thread,	NULL,	&downloadThread,	(void*)data );
}

ThemeDownloader::~ThemeDownloader()
{
	
}

const char *output;
GTimer *gTimer;
double last_size, last_time;

int progressFunc(GtkWidget *bar,
	     double t, /* dltotal */
	     double d, /* dlnow */
	     double ultotal,
	     double ulnow)
{
	gulong mSeconds;
	
	double time	=	g_timer_elapsed( gTimer,	&mSeconds );
	double dl_rate	=	(d - last_size) / (time - last_time);
	double eta	=	(t - d) / dl_rate;
	
	string speed	=	Get::getSize( (int)dl_rate ) + (string)"/s";
	string size	=	Get::getSize( t );
	
	string text = (string)"Downloading: " + Get::getUtf8( output ) + (string)"\n" +  Get::getSize( d ) + (string)"/" + 
			size + (string)" Speed: " + speed + (string)"\nETA: " + Get::getTime( (int)eta );
	
	gdk_threads_enter();
	gtk_progress_set_value( GTK_PROGRESS( bar ),	d*100.0/t );
	gtk_progress_bar_set_text( GTK_PROGRESS_BAR( bar ),	text.c_str() );
	gdk_threads_leave();
	
	last_size 		=	d;
	last_time		=	time;
	
	return 0;
}

size_t writeFunction( void *ptr,	size_t size,	size_t nmemb,	FILE *stream )
{
	return fwrite(ptr, size, nmemb, stream);
}
 
size_t readFunction( void *ptr,	size_t size,	size_t nmemb,	FILE *stream )
{
	return fread(ptr, size, nmemb, stream);
}

void ThemeDownloader::download( string url,	string name )
{
	CURL *conn	=	curl_easy_init();
	
	string outPath;
	string homeDir		=	g_get_home_dir();
	
	output	=	name.c_str();
	
	bool screenshot	=	false;
	bool ssExists		=	false;
	
	if( Util::compare( name,	"icon_theme.index" ) )
		outPath	=	homeDir	+	(string)"/.gbrowser/icon_theme.index";
	
	else if( g_str_has_suffix( name.c_str(),		".png" ) ||
			g_str_has_suffix( name.c_str(),	".jpg" )  ||
			g_str_has_suffix( name.c_str(),	".jpeg" ) )
	{
		screenshot	=	true;
		outPath		=	homeDir + (string)"/.gbrowser/thumbnails/" + name;
		ssExists		=	g_file_test( outPath.c_str(),	G_FILE_TEST_EXISTS );
	}
	
	else if( Util::checkWritePermissions( "/usr/share/icons" ) )
		outPath	=	"/usr/share/icons/" + name;

	else
		outPath	=	homeDir + (string)"/.gbrowser/icons/" + name;
		
	if( !ssExists )
	{
		FILE *out	=	fopen( outPath.c_str(),	"w" );

		last_size 		= 0;
		last_time 	= 0;
	
		gTimer	=	g_timer_new();
	
		g_timer_start( gTimer );

		curl_easy_setopt( conn,	CURLOPT_URL,					url.c_str() );
		curl_easy_setopt( conn,	CURLOPT_WRITEDATA,			out );
		curl_easy_setopt( conn,	CURLOPT_WRITEFUNCTION,		writeFunction );
		curl_easy_setopt( conn,	CURLOPT_READFUNCTION,		readFunction );
		curl_easy_setopt( conn,	CURLOPT_NOPROGRESS,		FALSE );
		curl_easy_setopt( conn,	CURLOPT_PROGRESSFUNCTION,	progressFunc );
		curl_easy_setopt( conn,	CURLOPT_PROGRESSDATA,		pBar );
	
		curl_easy_perform( conn );
	
		fclose( out );
	
		curl_easy_cleanup( conn );
	
		gdk_threads_enter();
		gtk_progress_set_value(		GTK_PROGRESS( pBar ),	0 );
		gtk_progress_bar_set_text(	pBar,	"Download Completed" );
		gdk_threads_leave();
	}
		
	if( Util::compare( name,	"icon_theme.index" ) )
	{
		gdk_threads_enter();
		this->appendThemes();
		gdk_threads_leave();
	}
	else if( screenshot )
	{
		int width,	height;
		gdk_pixbuf_get_file_info( outPath.c_str(),	&width,	&height );
		
		GdkPixbuf *buf;
		
		if( width <= 200	&&	height <= 200 )
			buf	=	gdk_pixbuf_new_from_file( outPath.c_str(),	NULL );
		else
			buf	=	gdk_pixbuf_new_from_file_at_size( outPath.c_str(),	200,	200,	NULL );
		
		gdk_threads_enter();
		gtk_image_set_from_pixbuf( preview,	buf );
		gdk_threads_leave();
		
		g_object_unref( G_OBJECT( buf ) );
	}
	else
	{	
		string cmd;	
		
		if( g_str_has_suffix( name.c_str(),	".tar.gz" ) )
			cmd	=	"tar zxf " + outPath + (string)" -C" + (string)g_path_get_dirname( outPath.c_str() );
		else if( g_str_has_suffix( name.c_str(),	".tar.bz2" ) )
			cmd	=	"tar jxf "+ outPath + (string)" -C" + (string)g_path_get_dirname( outPath.c_str() );

		system( cmd.c_str() );
		
		gdk_threads_enter();
		Settings::getInstance()->scanIconThemes();
		gdk_threads_leave();
	}
}

void *ThemeDownloader::downloadThread( void *arg )
{
	DownloadData *data	=	(DownloadData*)arg;
	ThemeDownloader::getInstance()->download( data->url,	data->name );
	delete data;
}

void ThemeDownloader::createThemeView()
{
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
	
	themeStore	=	gtk_list_store_new( 7,	G_TYPE_STRING,	G_TYPE_STRING, 
										G_TYPE_STRING,	G_TYPE_STRING,	
										G_TYPE_STRING,	G_TYPE_STRING,
						  				G_TYPE_STRING );
	themeView	=	GTK_TREE_VIEW( gtk_tree_view_new_with_model( GTK_TREE_MODEL( themeStore ) ) );
	
	col	=	gtk_tree_view_column_new();
	gtk_tree_view_column_set_title( col,	"Name" );
	gtk_tree_view_append_column( themeView,	col );
	renderer	=	gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start( col,	renderer,	TRUE );
	gtk_tree_view_column_add_attribute( col,	renderer,	"text",	0 );

	gtk_container_add( GTK_CONTAINER( view ),	GTK_WIDGET( themeView ) );
	
	gtk_tree_sortable_set_default_sort_func( GTK_TREE_SORTABLE( themeStore ), 	Util::sortFuncOnlyDirs,	NULL,	NULL );
	gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE( themeStore ), 
					      				GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
					      				GTK_SORT_ASCENDING );
	gtk_widget_show_all( view );
}

void ThemeDownloader::appendThemes()
{
	GKeyFile *file	=	g_key_file_new();
	string config	=	g_get_home_dir() + (string)"/.gbrowser/icon_theme.index";

	g_key_file_load_from_file( file,	config.c_str(),	G_KEY_FILE_NONE,	NULL );
	
	char **themeList	=	g_key_file_get_groups( file,	NULL );

	for( int it = 0;	themeList[it] != NULL;	it++ )
	{
		string name	=	themeList[it];
		string url		=	g_key_file_get_string( file,	name.c_str(),	"Url",	NULL );
		string author	=	g_key_file_get_string( file,	name.c_str(),	"Author",	NULL );
		string desc	=	g_key_file_get_string( file,	name.c_str(),	"Description",	NULL );
		string prev	=	g_key_file_get_string( file,	name.c_str(),	"PreviewImage",	NULL );
		string email	=	g_key_file_get_string( file,	name.c_str(),	"Email",	NULL );
		string filename	=	g_key_file_get_string( file,	name.c_str(),	"Name",	NULL );
		
		GtkTreeIter iter;

		gtk_list_store_append( themeStore,	&iter );
		gtk_list_store_set( themeStore,	&iter,	0,	name.c_str(),	1,	url.c_str(),
				    							2,	author.c_str(),	3,	desc.c_str(),
											4,	prev.c_str(),	5,	email.c_str(),	
											6,	filename.c_str(),	-1 );
	}

	g_key_file_free( file );
	g_strfreev( themeList );	
}

void ThemeDownloader::themeSelectionChanged( GtkTreeSelection *sel,	gpointer data )
{
	GtkTreeIter iter;
	gtk_tree_selection_get_selected( sel,	NULL,	&iter );
	
	const char *name,	*author,	*url,	*desc,	*prev,	*email;
	
	gtk_tree_model_get( GTK_TREE_MODEL( themeStore ),	&iter,	0,	&name,	1,	&url,
			    												2,	&author,	3,	&desc,
															4,	&prev,	5,	&email,	-1 );
	
	DownloadData *dData	=	new DownloadData;
	
	dData->url	=	prev;
	dData->name	=	g_path_get_basename( prev );
	
	pthread_create( &thread,	NULL,	&downloadThread,	(void*)dData );
	
	gtk_label_set_text( themeName,		name );
	gtk_label_set_text( authorLabel,		author );
	gtk_label_set_text( descriptionLabel,	desc );
	gtk_label_set_text( urlLabel,			url );
	gtk_label_set_text( emailLabel,		email );
}

void ThemeDownloader::downloadClicked( GtkWidget *widget,	gpointer data )
{
	GtkTreeIter iter;
	GtkTreeSelection *sel	=	gtk_tree_view_get_selection( themeView );
	
	gtk_tree_selection_get_selected( sel,	NULL,	&iter );
	
	const char *name,	*url;
	
	gtk_tree_model_get( GTK_TREE_MODEL( themeStore ),	&iter,	6,	&name,	1,	&url,		-1 );

	DownloadData *dData	=	new DownloadData;
	
	dData->url	=	url;
	dData->name	=	name;
	
	pthread_create( &thread,	NULL,	&downloadThread,	(void*)dData );
}

void ThemeDownloader::closeClicked( GtkWidget *widget,	gpointer data )
{
	gtk_widget_destroy( window );
}

void ThemeDownloader::onDestroyEvent( GtkWidget *widget,	gpointer data )
{
	delete themeDL;
	themeDL	=	NULL;	
}
